#!/usr/bin/perl
#
# Copyright (c) 2003-2004 Linuxant inc.
#
# NOTE: The use and distribution of this software is governed by the terms in
# the file LICENSE, which is included in the package. You must read this and
# agree to these terms before using or distributing this software.
# 

package dldrparser;

use 5.006;
use strict;

require Exporter;
our @ISA = qw{Exporter};
our %EXPORT_TAGS = ( 'all' => [ qw() ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
our @EXPORT = qw( );

our $VERSION = '0.10';

our $hidden = 0;
our $hwinst;
our %confFile;
our %strings;
our %driverConfig;
our $DEBUG = 1;

################################################################################

sub DEBUG {
	my ($msg) = @_;

	if ( $DEBUG ) {
		print "<pre>DEBUG: $msg</pre>";
	}
}

sub readConfig {
	my ($var, $val);
	open(CONFIG, "/usr/lib/driverloader/webconfdocs/includes/config.inc");
	while ( ($var,$val) = split(/=/, <CONFIG>) ) {
		chomp($val);
		$driverConfig{$var} = $val;
	}
	close(CONFIG);
	return %driverConfig;
}

sub error {
	my ($obj, $error) = @_;
	print STDERR "$error\n";
	exit 1;
}

sub ltrim {
	my ($inp) = @_;
	$inp =~ s/^\s+//;
	return($inp);
}

sub rtrim {
	my ($inp) = @_;
	$inp =~ s/\s+$//;
	return ($inp);
}

sub trim {
	my ($inp) = @_;
	$inp=ltrim($inp);
	$inp=rtrim($inp);
	return($inp);
}

sub strip_quotes {
	my ($inp) = @_;
	$inp =~ s/\"//g;
	return($inp);
}

sub nextSection {
	my ($section, $line);

	while ( $line = getLine("") ) {
		if ( $line =~ /^\[/ ) {
			$section = $line; chomp($section);
			$section =~ s/^\[//;
			$section =~ s/\].*//;
			$section =~ tr/A-Z/a-z/;
			return $section;
		}
	}
	return 0;
}

sub skipToSection {
	my ($obj, $section) = @_;
	my ($nextsection);

	seek(INF,0,0);
	$section =~ tr/A-Z/a-z/;
	while ( $nextsection = nextSection() ) {
		if ( $nextsection =~ /^$section(.nt)*$/i ) {
			return $nextsection;
		}
	}
	return 0;
}

sub lookFor {
	my ($obj, $string) = @_;
	my ($line, $pos);

	$pos = tell(INF);
	while ( $line = getLine("") ) {
		if ( $line =~ /^\[/ ) {
			seek(INF,$pos,0);
			return 0;
		}
		if ( $line =~ /^\s*$/ ) {
			next;
		}
		if ( $line =~ /$string/i ) {
			return $line;
		}
		$pos = tell(INF);
	}
	return 0;
}

sub upcase {
	my ($arg) = @_;
	$arg =~ tr/a-z/A-Z/;
	return $arg;
}

sub printIDs {
	my ($obj, $prefix) = @_;
	my ($nextLine);
	my ($cnt);

	$cnt = 0;
	while ( $nextLine = getLine("") ) {
		chomp($nextLine);
		if ( ($nextLine eq 0) || ($nextLine =~ /^\[/) ) {
			seek(INF,-length($nextLine),1);
			last;
		}
		if ( $nextLine =~ /PCI\\VEN_([[:xdigit:]]{4})&DEV_([[:xdigit:]]{4})([^&]|$)/i ) {
			print $prefix, "PCI ", upcase($1), " ", upcase($2), "\n";
			$cnt++;
		} elsif ( $nextLine =~ /PCI\\VEN_([[:xdigit:]]{4})&DEV_([[:xdigit:]]{4})&SUBSYS_([[:xdigit:]]{4})([[:xdigit:]]{4})/i ) {
			print $prefix, "PCI ", upcase($1), " ", upcase($2), " ", upcase($4), " ", upcase($3), "\n";
			$cnt++;
		} elsif ( $nextLine =~ /USB\\VID_([[:xdigit:]]{4})&PID_([[:xdigit:]]{4})([^&]|$)/i ) {
			print $prefix, "USB ", upcase($1), " ", upcase($2), "\n";
			$cnt++;
		}
	}
	return $cnt;
}

sub parseHwInst {
	my ($obj, $myhwinst) = @_;
	my (@parts, $VEN, $DEV, $SUBVEN, $SUBDEV, @match);

	$myhwinst =~ tr/a-z/A-Z/;
	@parts = split(/-/, $myhwinst);
	if ($parts[0] eq "PCI" ) {
		($VEN,$DEV) = split(/:/, $parts[2]);
		($SUBVEN, $SUBDEV) = split(/:/, $parts[3]);
		$match[0]="PCI\\\\VEN_$VEN&DEV_$DEV([^&]|\$)";
		$match[1]="PCI\\\\VEN_$VEN&DEV_$DEV&SUBSYS_$SUBDEV$SUBVEN";
#		$match[2]="PCI\\\\VEN_$VEN&DEV_$DEV&SUBSYS_$SUBVEN$SUBDEV";
	} elsif ($parts[0] eq "USB" ) {
		($VEN,$DEV) = split(/:/, $parts[3]);
		$match[0]="USB\\\\VID_$VEN&PID_$DEV([^&]|\$)";
	} else {
		$match[0]="HARDWARETYPEUNSUPPORTEDBYDRIVERLOADER";
	}
	return(@match);
}

sub findBestMatch {
	my ($obj, @match) = @_;
	my ($nextLine, $i, $bestMatch, @section);

	while ( $nextLine = getLine("") ) {
		chomp($nextLine);
		if ( ($nextLine eq 0) || ($nextLine =~ /^\[/) ) {
			seek(INF,-length($nextLine),1);
			last;
		}
		for ($i=0;$i<@match;$i++) {
			if ( $nextLine =~ /$match[$i]/i ) {
				$bestMatch = $match[$i];
				@section = split(/,/, $nextLine);
				$section[0] =~ s/.*=(\s)*//;
				$section[1] =~ s/^(\s)*//;
				$section[1] =~ s/(\s)*$//;
				if ( $i>0 ) {
					return @section;
				}
			}
		}
	}

	return(@section);
}

sub parseParam {
	my ($obj, $line) = @_;
	my (@params, $paramName, $paramDesc, $paramType, %type, $key, $val, @new);
	my (%min, %max, %step, %default, %limittext, %option, %label, $default);
	my ($disp, $x, $var);

	seek(INF,-length($line),1);

	while ( $line = getLine("") ) {

		if ($line =~ /^\s*$/) {
			next;
		}

		if ( $line =~ /HKR\s*,\s*defaults\s*,/i ) {
			next;
		}

		if ( ! ($line =~ /Ndi\\params\\$paramName/i) ) {
			seek(INF,-length($line),1);
			last;
		}

		@params = ($line =~ /(".*?"|[^,]+|,\s*,)/g);
		foreach my $index (0 .. $#params) { $params[$index] =~ s/^,\s*,$//; }

		$paramName = trim($params[1]);
		$paramName =~ s/Ndi\\params\\//i;
		$paramDesc = strip_quotes(trim($params[4]));
		$paramType = trim($params[2]);

		for ($paramType) {
			if (/type/i)	{ $type{$paramName} = $paramDesc; }
			elsif (/paramdesc/i) { $label{$paramName} = $paramDesc; }
			elsif (/min/i)	{ $min{$paramName} = $paramDesc; }
			elsif (/max/i)	{ $max{$paramName} = $paramDesc; }
			elsif (/step/i)	{ $step{$paramName} = $paramDesc; }
			elsif (/default/i)	{ $default{$paramName} = $paramDesc; }
			elsif (/limittext/i)	{ $limittext{$paramName} = $paramDesc; }
			else {
				if ( $paramName =~ /\\enum$/i ) {
					$paramName =~ s/\\enum//i;	
					push(@{$option{$paramName}}, trim($paramType));
					push(@{$option{$paramName}}, trim($paramDesc));
				} else {
					# print "<pre>NOT HANDLED: $line PARAM=:$paramType:</pre>";
				}
			}
		}
	}

	if ( defined($type{'IBSSGMode'}) ) {
		my (@new, %vals);

		@new = reverse @{$option{'IBSSGMode'}};
		while ( $key = pop(@new) ) {
			$val = pop(@new);
			$vals{strip_quotes(trim($key))} = $val;
		}
		if ( 	($type{'IBSSGMode'} eq "enum") &&
				($default{'IBSSGMode'} eq 0) &&
				($vals{"0"} eq "802.11b Only") &&
				($vals{"2"} eq "54g - Auto") ) {
			$default{'IBSSGMode'} = "2";
		}
	}

	if ( $hidden eq 0 ) {
		print "<tr><td>" . getString("", $label{$paramName}) . "</td>";
	}
	if ( $type{$paramName} eq "enum" ) {
		if ( defined($confFile{$paramName}) ) {
			$default = strip_quotes($confFile{$paramName});
		} else {
			$default = getString("", $default{$paramName});
			@new = reverse @{$option{$paramName}};
			if ( (grep{strip_quotes(trim($_)) eq $default}@new) eq 0 ) {
				($default) = sort @new;
				$default = strip_quotes(trim($default));
			}
		}
		if ( $hidden eq 1 ) {
			print "<input type=hidden name='$paramName' value='\"$default\"'>\n";
		} elsif ( $hidden eq 2 ) {
			print return($paramName,"\"$default\"");
		} else {
			print "<td><select name='$paramName'>\n";
			@new = reverse @{$option{$paramName}};
			while ( 1 ) {
				$val = pop(@new);
				if ( ! defined($val) ) {
					last;
				}
				$val = strip_quotes($val);
				$disp = pop(@new);
				print "<option value='\"$val\"'";
				if ( $val eq $default ) {
					print " selected";
				}
				print "> " . getString("", $disp) . "\n";
			}
			print "</select></td></tr>";
		}
	} elsif ( ($type{$paramName} eq "dword") || ($type{$paramName} eq "int") ) {
		if ( defined($confFile{$paramName}) ) {
			$default = strip_quotes($confFile{$paramName}); 
		} else {
			$default = getString("", $default{$paramName});
		}
		if ( $hidden eq 1 ) {
			print "<input type=hidden name='$paramName' value='\"$default\"'>\n";
		} elsif ( $hidden eq 2 ) {
			return($paramName,"\"$default\"");
		} else {
			print "<td><input name='$paramName' value='$default'>";
			if ( defined($min{$paramName}) && defined($max{$paramName}) ) {
				print "<br><i><font size=-1>Range: " . getString("", $min{$paramName})
					. " - " . getString("", $max{$paramName});
				if ( defined($step{$paramName}) && ($step{$paramName} ne 1)) {
					print " (Multiple of $step{$paramName})";
				}
				print "</font></i>";
			}
			print "</td></tr>";
		}
	} elsif ( $type{$paramName} eq "edit" ) {
		if ( defined($confFile{$paramName}) ) {
			$default = strip_quotes($confFile{$paramName}); 
		} else {
			$default = getString("", $default{$paramName});
		}
		if ( $hidden eq 1 ) {
			print "<input type=hidden name='$paramName' value='\"$default\"'>\n";
		} elsif ( $hidden eq 2 ) {
			print return($paramName,"\"$default\"");
		} else {
			print "<td><input name='::$paramName' value='$default' size=$limittext{$paramName} maxlength=$limittext{$paramName}></td></tr>";
		}
	}

	undef(%option); undef(%min); undef(%max);
	undef(%limittext);  undef(%label);  undef(%default);

	return;
}

sub FLG_ADDREG_BINVALUETYPE() { 0x00000001 }
sub FLG_ADDREG_NOCLOBBER() { 0x00000002 }
sub FLG_ADDREG_DELVAL() { 0x00000004 }
sub FLG_ADDREG_APPEND() { 0x00000008 }
sub FLG_ADDREG_KEYONLY() { 0x00000010 }
sub FLG_ADDREG_OVERWRITEONLY() { 0x00000020 }
sub FLG_ADDREG_64BITKEY() { 0x00001000 }
sub FLG_ADDREG_KEYONLY_COMMON() { 0x00002000 }
sub FLG_ADDREG_32BITKEY() { 0x00004000 }
sub FLG_ADDREG_TYPE_SZ() { 0x00000000 }
sub FLG_ADDREG_TYPE_MULTI_SZ() { 0x00010000 }
sub FLG_ADDREG_TYPE_EXPAND_SZ() { 0x00020000 }
sub FLG_ADDREG_TYPE_DWORD() { 0x00010001 }
sub FLG_ADDREG_TYPE_NONE() { 0x00020001 }

sub doQuote {
	my ($val, $flags) = @_;

	if ($flags == FLG_ADDREG_BINVALUETYPE) {
		return $val;
	}
	$val = getString("", strip_quotes($val));
	if ( (($flags & FLG_ADDREG_TYPE_DWORD) != FLG_ADDREG_TYPE_DWORD) ) {
		$val = "\"$val\"";
	}
	return $val;
}

sub parseStaticParam {
	my ($obj, $line) = @_;
	my ($regroot, $subkey, $flags, $var, $val);
	my (@fields);

	@fields = ($line =~ /(".*?"|[^,]+|,\s*,)/g);
	foreach my $index (0 .. $#fields) { $fields[$index] =~ s/^,\s*,$//; }
	$regroot=shift(@fields);
	$subkey=shift(@fields);
	$var=shift(@fields);
	$flags=shift(@fields);
	if ($flags == FLG_ADDREG_BINVALUETYPE) {
		if (($#fields + 1) == 4) {
			$val= "0x" . trim($fields[3]) . trim($fields[2]). trim($fields[1]). trim($fields[0]);
		} elsif (($#fields + 1) == 2) {
			$val= "0x" . trim($fields[1]). trim($fields[0]);
		} elsif (($#fields + 1) == 1) {
			$val= "0x" . trim($fields[0]);
		} else  {
			$val="{";
			while($#fields >= 0) {
				$val .= trim(shift(@fields));
				if($#fields >= 0) {
					$val .= ", ";
				}
			}
			$val .= "}";
		}
		if($val =~ /^0x/i) {
			$val =~ s/^0x0+/0x/i;
			if(oct($val) < 10) { # get rid of unnecessary 0x
				$val = oct($val);
			}
		}
	} else {
		$val=shift(@fields);
	}
	$flags = oct($flags) if $flags =~ /^0/;
	$var = getString("", strip_quotes(trim($var)));
	$val = doQuote(trim($val), $flags);
	if ( defined($confFile{$var}) ) {
			$val = $confFile{$var}; 
	}

	# substitute strings
	while($val =~ m/(%\w+%)/g) {
		my($token);
		$token=$1;
		if (getString("", "$token") ne "$token") {
			$val =~ s/$token/getString("", "$token")/ge;
		}
	}

	if ( $hidden eq 2 ) {
		return($var,$val);
	} else {
		print "<input type=hidden name='$var' value='$val'>\n";
	}
}

sub readStrings {
	my ($string, $line, $token);

	skipToSection("", "strings");
	while ( $line = getLine("") ) {
		if ($line =~ /^\s*$/) {
			next;
		}
		if ( ! ($line =~ /^\[/) ) {
			chomp($line);
			($token, $string) = split(/=/, $line);
			$token = trim($token);
			$string = strip_quotes(trim($string));
			$strings{"%" . $token . "%"} = $string;
		} else {
			last;
		}
	}
	seek(INF,0,0);
	return %strings;
}

sub getString {
	my ($obj, $string) = @_;
	if ( exists($strings{$string}) ) {
		return $strings{$string};
	} else {
		return $string;
	}
}

sub readConfigFile {
	my ($var, $val, $mac);

	open(CNF, "$driverConfig{'cnfdir'}/$hwinst");
	while ( ($var,$val) = split(/=/, <CNF>) ) {
		$var = strip_quotes(trim($var));
#		$val = strip_quotes(trim($val));
		$val = trim($val);
		$confFile{$var} = $val;
	}
	close(CNF);

	$mac = readlink("$driverConfig{'cnfdir'}/$hwinst");

	open(LIC, "$driverConfig{'nvmdir'}/$mac");
	while ( ($var, $val) = split(/=/, <LIC>) ) {
		$var = strip_quotes(trim($var));
		$val = strip_quotes(trim($val));
		$confFile{$var} = $val;
	}
	close(LIC);

	return %confFile;
}

sub convLicense {
	my ($obj, $lic) = @_;
	my ($md5);

	$lic =~ s/^0x//i;
	open(MD5, "/bin/echo $lic | /usr/bin/md5sum |");
	$md5 = substr(<MD5>, 0, 4);
	$md5 =~ tr/a-z/A-Z/;
	close(MD5);
	$lic .= $md5;
	$lic =~ s/(..)(..)(..)(..)(..)(..)/$1-$2-$3-$4-$5-$6/;
	return($lic);
}

sub configFile {
	my ($obj, $config) = @_;
	open(INF, $config) || error("", "Can not open file '$config'.");
}

sub getDeviceSection {
	my ($line, $vendor, $devSection, $sectionBasename);

	skipToSection("", "Manufacturer");
	while ( $line = getLine("") ) {
		if (!($line =~ /^\s*$/)) {
			last;
		}
	}

	if ( $line =~ /.*\=.*/ ) {
		($vendor, $devSection) = split(/=/, $line);
		$vendor=getString("", trim($vendor));
		$devSection=trim($devSection);
		$devSection =~ s/[ 	]*//g;
	} else {
		$vendor=getString("", trim($line));
		$devSection=$vendor;
	}

	if ( $devSection =~ /^.*\,NT/i ) {
		($sectionBasename) = split(/,/, $devSection);
		if ( $devSection =~ /NTx86.5.1/i ) {
			return "$sectionBasename.NTx86.5.1";
		} elsif ( $devSection =~ /NTx86/i ) {
			return "$sectionBasename.NTx86";
		} elsif ( $devSection =~ /NT.5.1/i ) {
			return "$sectionBasename.NT.5.1";
		} else {
			return "$sectionBasename.NT";
		} 
	}

	return "$devSection";
}

sub getBaseName {
	my ($basename, $nt_match, $line);

	skipToSection("", "Version");

	$basename = ""; $nt_match = 0;
	while ( $line = lookFor("", "CatalogFile") ) {
		if ( $line =~ /catalogfile.ntx86\s*=/i ) {
			$basename = trim($line);
			last;
		} elsif ( $line =~ /catalogfile.nt\s*=/i ) {
			$nt_match = 1;
			$basename = trim($line);
		} elsif ( ($line =~ /catalogfile\s*=/i) && !$nt_match ) {
			$basename = trim($line);
		}
	}

	if ( $basename ne "" ) {
		$basename =~ tr/A-Z/a-z/;
		$basename =~ s/^\s*catalogfile.*\s*=\s*(.*)\.cat/$1/;
	} else {
		$basename = getSysName();
		$basename =~ s/\.sys$//i;
	}

	return $basename;
}

sub getSysName {
	my ($x, $line, @match, $devsection, $sysfile);

	$devsection = getDeviceSection();
	skipToSection("", $devsection);
	if ( defined($hwinst) && ($hwinst ne "") ) {
		@match = parseHwInst("", $hwinst);
		($line, $x) = findBestMatch("", @match);
	} else {
		while ( $line = getLine() ) {
			if (!($line =~ /^\s*$/)) {
				($x,$line) = split(/=/, $line);
				($line) = split(/,/, trim($line));
				$line = trim($line);
				last;
			}
		}
	}
	skipToSection("", $line . ".ntx86.Services") || skipToSection("", $line . ".nt.Services") || skipToSection("", $line . ".Services");
	$line = lookFor("", "AddService");
	($x,$line) = split(/=/, $line);
	$line = trim($line);
	($x,$x,$line) = split(/,/, $line);
	$line = trim($line);
	skipToSection("",$line);
	$line = lookFor("","ServiceBinary");
	($x,$line) = split(/=/, $line);
	$line = trim($line);

	# substitute strings
	while($line =~ m/(%\w+%)/g) {
		my($token);
		$token=$1;
		if (getString("", "$token") ne "$token") {
			$line =~ s/$token/getString("", "$token")/ge;
		}
	}

	if ( $line =~ /\\/ ) {
		$line =~ s/.*\\(.*)\.sys/$1/i;
	} else {
		$line =~ s/(.*)\.sys/$1/i;
	}
	$sysfile = $line . ".sys";
	$sysfile =~ tr/A-Z/a-z/;

	return $sysfile;
}

sub getRegSections {
	my($regSections, @reg, @unique, $x, %reg_exists);

	$regSections = lookFor("", "AddReg");
	chomp($regSections);
	@reg = split(/,/, $regSections);
	($x,$reg[0]) = split(/=/, $reg[0]);
	$reg[0] = trim($reg[0]);
	
	foreach $x (reverse @reg) {
		$x = trim($x);
		if ( ! $reg_exists{$x} ) {
			$reg_exists{$x} = 1;
			push(@unique, $x);
		}
	}

	@reg = reverse(@unique);
	
	return @reg;
}

sub getSrcFiles {
	my ($obj) = @_;
	my ($file, $line, @files);

	skipToSection("","SourceDisksFiles.NTX86") || skipToSection("","SourceDisksFiles.NT") || skipToSection("","SourceDisksFiles.X86") || skipToSection("","SourceDisksFiles");
	while ( $line = getLine() ) {
		if ($line =~ /^\s*$/) {
			next;
		} elsif ( $line =~ /^\[/ ) {
			last;
		} else {
			($file) = split(/=/, trim($line));
			$file   =~ tr/A-Z/a-z/;
			$file   = trim($file);
			if ( ! ($file =~ /dll$/) &&
					! ($file =~ /exe$/) &&
					! ($file =~ /inf$/) &&
					! ($file =~ /usb8023[mw]\.sys$/) &&
					! ($file =~ /rndismp[mw]\.sys$/) &&
					! ($file =~ /zd11u(98|me)\.sys$/) &&
					! ($file =~ /vnetu9xl\.sys$/) &&
					! ($file =~ /9x\.sys$/) ) {
				push(@files, $file);
			}
		}
	}

	return @files;
}

sub getFileSubs {
	my ($obj, $section, $sysfile) = @_;
	my ($x,$line,@sec, $sec, $src, $dst);

	skipToSection("", $section . ".ntx86") || skipToSection("", $section . ".nt") || skipToSection("", $section);
	if ( $line = lookFor("", "CopyFiles") ) {
		($x,$line) = split(/=/, $line);
		@sec = split(/,/, $line);
		foreach $sec ( @sec ) {
			skipToSection("", trim($sec));
			if ( $line = lookFor("", $sysfile) ) {
				($dst, $src) = split(/,/, $line);
				$src =~ tr/A-Z/a-z/; $src = trim($src);
				$dst =~ tr/A-Z/a-z/; $dst = trim($dst);
				if ( ($dst ne $src) && ($src ne "") ) {
					$sysfile = $src;
				}
			}
		}
	}
	return($sysfile);
}

sub setHwInst {
	my ($obj, $hwinststr) = @_;
	$hwinst = $hwinststr;
	return $hwinst;
}

sub getLine {
	my ($line);

	$line = <INF>;
	$line =~ s/\000//g; # workaround for unicode
	$line =~ s/^\#.*//; # workaround for WUSB54G.INF version 1.0.5.0
	$line =~ s/\;.*//;
	return ($line);
}

sub setHidden {
	my ($obj, $adv) = @_;
	$hidden = $adv;
}

################################################################################

sub new {
	my ($self) = {};
	bless($self);
	return $self;
}

1;
