/*
 * Copyright (c) 2003-2004 Linuxant inc.
 *
 * NOTE: The use and distribution of this software is governed by the terms in
 * the file LICENSE, which is included in the package. You must read this and
 * agree to these terms before using or distributing this software.
 *
 */

#define OS_NOSTRINGREDEFS

#include "oscompat.h"
#include "osservices.h"
#include "ndiswrap.h"
#include "netndis.h"

#include <linux/pci.h>
#include <linux/usb.h>
#include <linux/kmod.h>
#include <linux/ctype.h>

MODULE_AUTHOR("Copyright (C) 2003-2004 Linuxant inc.");
MODULE_DESCRIPTION(DLDRDESC);
MODULE_LICENSE("see LICENSE file; Copyright (c)2003-2004 Linuxant inc.");
MODULE_INFO(supported, "yes");

extern struct pci_driver dldrpci_driver;
#if defined(DLDR_USB) && (defined(CONFIG_USB) || defined(CONFIG_USB_MODULE))
extern struct usb_driver dldrusb_driver;
#endif

int dldr_pci_registered;
int dldr_usb_registered;

DECLARE_MUTEX(dldr_init_mutex);

static int __init dldr_init (void)
{
	int err;
	int errpci;
#if defined(DLDR_USB) && (defined(CONFIG_USB) || defined(CONFIG_USB_MODULE))
	int errusb;
#endif
    char *argv[4], *envp[3];

	OsTestPrintf("%s\n", __FUNCTION__);

    envp [0] = "HOME=/";
    envp [1] = "PATH=/sbin:/bin:/usr/sbin:/usr/bin:"DLDRSBINDIR;
    envp [2] = 0;
    argv[0] = DLDRSBINDIR"/dldrconfig";
    argv[1] = "--auto";
    argv[2] = "--mod-pre-init";
    argv[3] = 0;

	err = OsForkWait(argv[0], argv, envp);
    if(err) {
        printk(KERN_ERR"%s: OsForkWait %s returned %d\n", __FUNCTION__, argv[0], err);
    }

	netndis_init();
	err = NdisWrapInit();
	if(err) {
		netndis_exit();
		return err;
	}
	
	errpci = pci_module_init (&dldrpci_driver);
	if(!errpci)
		dldr_pci_registered = 1;
#if defined(DLDR_USB) && (defined(CONFIG_USB) || defined(CONFIG_USB_MODULE))
	errusb = usb_register(&dldrusb_driver);
	if(!errusb)
		dldr_usb_registered = 1;
#endif

    envp [0] = "HOME=/";
    envp [1] = "PATH=/sbin:/bin:/usr/sbin:/usr/bin:"DLDRSBINDIR;
    envp [2] = 0;
    argv[0] = DLDRSBINDIR"/dldrconfig";
    argv[1] = "--auto";
    argv[2] = "--mod-init";
    argv[3] = 0;

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
    err = call_usermodehelper(argv[0], argv, envp);
#else
    err = call_usermodehelper(argv[0], argv, envp, 1);
#endif
    if(err && (err != SIGTERM)) {
        printk(KERN_ERR"%s: call_usermodehelper %s returned %d\n", __FUNCTION__, argv[0], err);
    }

	if(!dldr_pci_registered && !dldr_usb_registered) {
		NdisWrapExit();
		netndis_exit();
		return errpci;
	}
	
	return 0;
}


static void __exit dldr_cleanup (void)
{
	OsTestPrintf("%s\n", __FUNCTION__);

#if defined(DLDR_USB) && (defined(CONFIG_USB) || defined(CONFIG_USB_MODULE))
	if(dldr_usb_registered) {
		dldr_usb_registered = 0;
		NdisWrapNotifyBusDeregister(OSDEVTYPE_USB);
		usb_deregister (&dldrusb_driver);
	}
#endif
	if(dldr_pci_registered) {
		dldr_pci_registered = 0;
		NdisWrapNotifyBusDeregister(OSDEVTYPE_PCI);
		pci_unregister_driver (&dldrpci_driver);
	}
	NdisWrapExit();
	netndis_exit();
}

module_init(dldr_init);
module_exit(dldr_cleanup);
