/*
 * Copyright (c) 2003-2004 Linuxant inc.
 *
 * NOTE: The use and distribution of this software is governed by the terms in
 * the file LICENSE, which is included in the package. You must read this and
 * agree to these terms before using or distributing this software.
 *
 */

#define OS_NOSTRINGREDEFS

#include "oscompat.h"

#if defined(CONFIG_USB) || defined(CONFIG_USB_MODULE)
#include "osservices.h"
#include "osusb.h"
#include "ndiswrap.h"
#include "netndis.h"

#include <linux/usb.h>
#include <linux/kmod.h>
#include <linux/ctype.h>

static struct usb_device_id dldrusb_tbl[] =
{
	// Conexant PRISM:
	{ USB_DEVICE(0x0506, 0x0A11) }, // 3COM 3CRWE254G72
	{ USB_DEVICE(0x050D, 0x7050) }, // Belkin F5D7050
	{ USB_DEVICE(0x124A, 0x4023) }, // Blitzz BWU713, IOGEAR GWU513, Shuttle PN15 (Airvast WM168g)
	{ USB_DEVICE(0x2001, 0x3700) }, // D-Link DWL-122
	{ USB_DEVICE(0x2001, 0x3701) }, // D-Link DWL-G120
	{ USB_DEVICE(0x2001, 0x3703) }, // D-Link DWL-G122
	{ USB_DEVICE(0x1915, 0x2234) }, // Linksys WUSB54G
	{ USB_DEVICE(0x1915, 0x2235) }, // Linksys WUSB54GP
	{ USB_DEVICE(0x5041, 0x2234) }, // Linksys WUSB54G
	{ USB_DEVICE(0x5041, 0x2235) }, // Linksys WUSB54GP
	{ USB_DEVICE(0x0846, 0x4200) }, // Netgear WG121
	{ USB_DEVICE(0x0846, 0x4210) }, // Netgear WG121
	{ USB_DEVICE(0x0846, 0x4220) }, // Netgear WG111
	{ USB_DEVICE(0x0846, 0x4110) }, // Netgear MA111
	{ USB_DEVICE(0x0707, 0xEE06) }, // SMC 2862W-G
	{ USB_DEVICE(0x182D, 0x096B) }, // Sitecom WL-107
	{ USB_DEVICE(0x0CDE, 0x0006) },	// Medion 40900
	{ USB_DEVICE(0x124A, 0x168B) }, // WM168
	{ USB_DEVICE(0x083A, 0x4531) }, // T-Com Sinus 154 data II
	{ USB_DEVICE(0x0BAF, 0x0118) }, // U.S. Robotics USR5422
	{ USB_DEVICE(0x09AA, 0x1000) }, // Generic?

	// Broadcom BCM94333U:
	{ USB_DEVICE(0x0411, 0x004B) }, // Buffalo WLI-USB-G54
	{ USB_DEVICE(0x0BAF, 0x0111) }, // U.S. Robotics USR5420

	// ZyDAS:
	{ USB_DEVICE(0x0ACE, 0x1211) }, // ZyDAS ZD1211; Blitzz BWU723, Airlink+ AWLL3025
	{ USB_DEVICE(0x0586, 0x3401) }, // ZyXEL G-220

	// Intel:
	{ USB_DEVICE(0x2474, 0x3642) }, // Intersil
	{ USB_DEVICE(0x8086, 0x1110) }, // Intel
	{ USB_DEVICE(0x8086, 0x1111) }, // Intel Windansea
	{ USB_DEVICE(0x049F, 0x0033) }, // Compaq
	{ USB_DEVICE(0x05E0, 0x0800) }, // Symbol Windansea

	{ .match_flags = USB_DEVICE_ID_MATCH_DEV_CLASS, .bDeviceClass = USB_CLASS_COMM },
	{ .match_flags = USB_DEVICE_ID_MATCH_DEV_CLASS, .bDeviceClass = USB_CLASS_VENDOR_SPEC },

#if 0 // experimental
	// Atmel:
	{ USB_DEVICE(0x077B, 0x2219) }, // Linksys WUSB11 V2.6 802.11b Adapter
	{ .match_flags = USB_DEVICE_ID_MATCH_DEV_CLASS, .bDeviceClass = USB_CLASS_APP_SPEC },
#endif

	{0,}                    /* terminate list */
};


MODULE_DEVICE_TABLE(usb, dldrusb_tbl);

extern struct semaphore dldr_init_mutex;

static void __devexit dldrusb_disconnect (
#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) )
		struct usb_device *udev, void *ptr
#else
		struct usb_interface *intf
#endif
		)
{
#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) )
	POS_DEVNODE pDevNode = ptr;
#else
	POS_DEVNODE pDevNode = usb_get_intfdata(intf);
	struct usb_device *udev = interface_to_usbdev(intf);
#endif
	extern int dldr_usb_registered;
	struct net_device *ndev;
	int needsUsbReset = 0;
	ULONG wrapState;

	OsTracePrintf("%s: pDevNode=%p\n", __FUNCTION__, pDevNode);

	if(!pDevNode)
		return;

	if(pDevNode->hwDriverName && !strcmp(pDevNode->hwDriverName, "zd1211u"))
		needsUsbReset = 1;

	wrapState = NdisWrapEnter();

	NdisWrapNotifyAdapterRemoved(pDevNode->miniportAdapter);

	down(&dldr_init_mutex);
	ndev = pDevNode->netDev;
	pDevNode->netDev = NULL;

	if(ndev) {
		unregister_netdev(ndev);
	}

	NdisWrapHaltAdapter(pDevNode->miniportAdapter);

#if ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0) )
	usb_set_intfdata (intf, NULL);
#endif

	if(ndev) {
		free_netdev(ndev);
	}
	NdisWrapDestroyAdapter(pDevNode->miniportAdapter);
	OsUsbDestroyInstance(pDevNode->hwDev);

	if(needsUsbReset && !dldr_usb_registered) { // still physically connected?
		// some devices (ZD1211) need a bus reset to restart properly
		int err = usb_reset_device(udev);
		if (err)
			printk(KERN_ERR "%s: Failed to reset device %s (%d)\n", DLDRNAME, pDevNode->hwInstName, err);
	}
	up(&dldr_init_mutex);

	NdisWrapLeave(wrapState);

	kfree(pDevNode);
}


#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) )
static void * __devinit dldrusb_probe(
		struct usb_device *udev, unsigned int ifnum,
		const struct usb_device_id *id)
#else
static int __devinit dldrusb_probe(struct usb_interface *intf,
		const struct usb_device_id *id)
#endif
{
	int err;
	POS_DEVNODE pDevNode;
	struct net_device *ndev = NULL;
	char buf[100];
    char *argv[3], *envp[3];
#if ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,20) )
    char *p;
	u_int8_t csz;
#endif
#if ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0) )
	struct usb_device *udev = interface_to_usbdev(intf);
	//__u8 ifnum = intf->altsetting->desc.bInterfaceNumber;
#endif
	ULONG wrapState;

	OsTracePrintf("%s udev=%p\n", __FUNCTION__, udev);

	pDevNode = kmalloc(sizeof(*pDevNode), GFP_KERNEL);
	if(!pDevNode) {
#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) )
		return NULL;
#else
		return -ENOMEM;
#endif
	}

	down(&dldr_init_mutex);

	wrapState = NdisWrapEnter();

	memset(pDevNode, 0, sizeof(*pDevNode));
	pDevNode->hwDev = OsUsbCreateInstance(pDevNode, udev);
	pDevNode->hwModule = THIS_MODULE;
	pDevNode->hwBusType = OSDEVTYPE_USB;

	if(!pDevNode->hwDev) {
		err = -ENOMEM;
		goto outerr;
	}
	
#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,4,20) )
	snprintf(pDevNode->hwInstName, sizeof(pDevNode->hwInstName), "USB-0000:00:00.0-%d:%d-%04X:%04X", udev->bus->busnum, udev->devnum, udev->descriptor.idVendor, udev->descriptor.idProduct);
#else
	/* count : chars in bus_name to determine if fake domain_nr must be added
	 * to make 2.4 instance names identical to 2.6 */
	for(p = udev->bus->bus_name, csz = 0; *p; p++)
		if(*p == ':')
			csz++;
	snprintf(pDevNode->hwInstName, sizeof(pDevNode->hwInstName), "USB-%s%s-%s:%d-%04X:%04X", csz == 1 ? "0000:" : "", udev->bus->bus_name, udev->devpath, udev->devnum, udev->descriptor.idVendor, udev->descriptor.idProduct);
#endif

	err = NdisWrapCreateAdapter(pDevNode, DLDRVARDIR, &pDevNode->miniportAdapter);
	OsTracePrintf("%s: NdisWrapCreateAdapter returned %d miniportAdapter=%p instname=%s pDevNode=%p\n", __FUNCTION__, err, pDevNode->miniportAdapter, pDevNode->hwInstName, pDevNode);
	if(err) {
		pDevNode->miniportAdapter = NULL;
		goto outerr;
	}

#if ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0) )
	usb_set_intfdata (intf, pDevNode);
#endif

	ndev = alloc_netndisdev(pDevNode);
	if(!ndev) {
#if ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0) )
		usb_set_intfdata (intf, NULL);
#endif
		err = -ENOMEM;
		goto outerr;
	}

	SET_MODULE_OWNER(ndev);
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
	SET_NETDEV_DEV(ndev, &intf->dev);
#endif

	pDevNode->netDev = ndev;

	err = register_netdev(ndev);
	if (err) {
		printk(KERN_ERR "%s: Failed to register net device (%d)\n", ndev->name, err);
		pDevNode->netDev = NULL;
		free_netdev(ndev);
#if ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0) )
		usb_set_intfdata (intf, NULL);
#endif
		goto outerr;
	}

	buf[0] = '\0';
	NdisWrapGetVendorDesc(pDevNode->miniportAdapter, buf, sizeof(buf));

	NdisWrapLeave(wrapState);

	up(&dldr_init_mutex);

	printk(KERN_DEBUG"%s:%s%s at %s%s%s (MAC address %02X:%02X:%02X:%02X:%02X:%02X) ready\n", ndev->name,
			buf[0] ? " ":"", buf,
#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,4,20) )
			pDevNode->hwInstName, "", "", 
#else
			udev->bus->bus_name, "", udev->devpath, 
#endif
			ndev->dev_addr[0], ndev->dev_addr[1], ndev->dev_addr[2],
			ndev->dev_addr[3], ndev->dev_addr[4], ndev->dev_addr[5]);

    envp [0] = "HOME=/";
    envp [1] = "PATH=/sbin:/bin:/usr/sbin:/usr/bin:"DLDRSBINDIR;
    envp [2] = 0;

    argv[0] = DLDRSBINDIR"/dldrconfig";
    argv[1] = "--auto";
    sprintf(buf, "--netif=%s", ndev->name);
    argv[2] = buf;
    argv[2] = "--modalias";
    argv[3] = 0;

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
    err = call_usermodehelper(argv[0], argv, envp);
#else
    err = call_usermodehelper(argv[0], argv, envp, 1);
#endif
    if(err && (err != SIGTERM)) {
        printk(KERN_ERR"%s: call_usermodehelper %s returned %d\n", __FUNCTION__, argv[0], err);
    }

#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) )
	return pDevNode;
#else
	return 0;
#endif

outerr:
	if(pDevNode->miniportAdapter)
		NdisWrapDestroyAdapter(pDevNode->miniportAdapter);
	if(pDevNode->hwDev)
		OsUsbDestroyInstance(pDevNode->hwDev);

	NdisWrapLeave(wrapState);
	up(&dldr_init_mutex);
	kfree(pDevNode);
#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) )
	return NULL;
#else
	return err;
#endif
}

struct usb_driver dldrusb_driver = {
	.name           = "driverloader",
	.id_table       = dldrusb_tbl,
#if ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,20) )
	.owner			= THIS_MODULE,
#endif
	.probe          = dldrusb_probe,
	.disconnect     = __devexit_p(dldrusb_disconnect),
};
#endif // defined(CONFIG_USB) || defined(CONFIG_USB_MODULE)
