/*
 * Copyright (c) 2003-2004 Linuxant inc.
 * 
 * 1.   Permitted use. Redistribution and use in source and binary forms,
 * without modification, are only permitted under the terms set forth herein.
 * 
 * 2.   Disclaimer of Warranties. LINUXANT, ITS SUPPLIERS, AND OTHER CONTRIBUTORS
 * MAKE NO REPRESENTATION ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.
 * IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTIES OF ANY KIND.
 * LINUXANT AND OTHER CONTRIBUTORS DISCLAIMS ALL WARRANTIES WITH REGARD
 * TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, GOOD TITLE AND AGAINST INFRINGEMENT.
 * 
 * This software has not been formally tested, and there is no guarantee that
 * it is free of errors including, but not limited to, bugs, defects,
 * interrupted operation, or unexpected results. Any use of this software is
 * at user's own risk.
 * 
 * 3.   No Liability.
 * 
 * (a) Linuxant, its suppliers, or contributors shall not be responsible for
 * any loss or damage to users, customers, or any third parties for any reason
 * whatsoever, and LINUXANT, ITS SUPPLIERS OR CONTRIBUTORS SHALL NOT BE LIABLE
 * FOR ANY ACTUAL, DIRECT, INDIRECT, SPECIAL, PUNITIVE, INCIDENTAL, OR
 * CONSEQUENTIAL (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED, WHETHER IN CONTRACT, STRICT OR OTHER LEGAL THEORY OF
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 * 
 * (b) User agrees to hold Linuxant, its suppliers, and contributors harmless
 * from any liability, loss, cost, damage or expense, including attorney's fees,
 * as a result of any claims which may be made by any person, including
 * but not limited to User, its agents and employees, its customers, or
 * any third parties that arise out of or result from the manufacture,
 * delivery, actual or alleged ownership, performance, use, operation
 * or possession of the software furnished hereunder, whether such claims
 * are based on negligence, breach of contract, absolute liability or any
 * other legal theory.
 * 
 * 4.   Notices. User hereby agrees not to remove, alter or destroy any
 * copyright, trademark, credits, other proprietary notices or confidential
 * legends placed upon, contained within or associated with the Software,
 * and shall include all such unaltered copyright, trademark, credits,
 * other proprietary notices or confidential legends on or in every copy of
 * the Software.
 * 
 * 5.   Reverse-engineering. User hereby agrees not to reverse engineer,
 * decompile, or disassemble the portions of this software provided solely
 * in object form, nor attempt in any manner to obtain their source-code.
 * 
 * 6.   Redistribution. Redistribution of this software is only permitted
 * for exact copies (without modification) of versions explicitly marked
 * and officially released by Linuxant with the word "free" in their name.
 * Redistribution or disclosure of other versions, derivatives or license key
 * information is expressly prohibited without explicit written approval signed
 * by an authorized Linuxant officer.
 * 
 * 7.   Performance. Actual speeds vary and are often less than the maximum
 * possible.
 * 
 * 
 */

#ifndef _DLDR_NDISWRAP_H_
#define _DLDR_NDISWRAP_H_

#define UMAC2STR(a) ((PUCHAR)a)[0], ((PUCHAR)a)[1], ((PUCHAR)a)[2], ((PUCHAR)a)[3], ((PUCHAR)a)[4], ((PUCHAR)a)[5]
#define UMACSTR "%02X:%02X:%02X:%02X:%02X:%02X"

#if (defined(DLDR_TEST) || defined(DLDR_DEBUG)) && !defined(DLDR_ASSERTS)
#define DLDR_ASSERTS
#endif

#ifdef DLDR_ASSERTS
#define DLDR_ASSERT(exp) ((!(exp)) ? (OsErrorPrintf("%s: Assertion %s failed at %s:%d (Current IRQL=%s)\n", __FUNCTION__, #exp, __FILE__, __LINE__, OsIrqlName(OsGetIrql())),/*OsCalledFrom(__builtin_return_address(0)),*/OsDumpStack(),FALSE) : TRUE)
#else
#define DLDR_ASSERT(exp) do {} while (0)
#endif

typedef enum {
    NDISWRAP_NETWORKTYPE_FH,
    NDISWRAP_NETWORKTYPE_DS,
    NDISWRAP_NETWORKTYPE_OFDM5,
    NDISWRAP_NETWORKTYPE_OFDM24
} ndiswrapnetworktype_t;

typedef enum {
    NDISWRAP_AUTHMODE_OPEN,
    NDISWRAP_AUTHMODE_SHARED,
    NDISWRAP_AUTHMODE_AUTOSWITCH,
    NDISWRAP_AUTHMODE_WPA,
    NDISWRAP_AUTHMODE_WPAPSK,
    NDISWRAP_AUTHMODE_WPANONE
} ndiswrapauthmode_t;

/* compatible with IW_MODE_ defines */
typedef enum {
	NDISWRAP_IWMODE_AUTO,
	NDISWRAP_IWMODE_ADHOC,
	NDISWRAP_IWMODE_INFRA
} ndiswrapiwmode_t;


typedef enum {
    NDISWRAP_ENCRYPTION1ENABLED,
    NDISWRAP_ENCRYPTIONDISABLED,
    NDISWRAP_ENCRYPTION1KEYABSENT,
    NDISWRAP_ENCRYPTIONNOTSUPPORTED,
    NDISWRAP_ENCRYPTION2ENABLED,
    NDISWRAP_ENCRYPTION2KEYABSENT,
    NDISWRAP_ENCRYPTION3ENABLED,
    NDISWRAP_ENCRYPTION3KEYABSENT
} ndiswrapencryption_t;

typedef enum {
	NDISWRAP_PWRMODE_CAM,
	NDISWRAP_PWRMODE_MAX_PSP,
	NDISWRAP_PWRMODE_FAST_PSP
} ndiswrappowermode_t;

typedef enum {
	NDISWRAP_PWRSTATE_UNKNOWN,
	NDISWRAP_PWRSTATE_D0,
	NDISWRAP_PWRSTATE_D1,
	NDISWRAP_PWRSTATE_D2,
	NDISWRAP_PWRSTATE_D3
} ndiswrappowerstate_t;

struct net_device;
struct sk_buff;

typedef
__shimcall__
void *
(*NDISWRAP_RX_HANDLER)(
    struct net_device *dev,
	int status,
	void *packetctx,
	unsigned int headersize,
	void *buffer,
	unsigned int length
    );

typedef enum {
	NDISWRAP_LINKSTATUS_NOT_CONNECTED,
	NDISWRAP_LINKSTATUS_CONNECTED,
	NDISWRAP_LINKSTATUS_DISCONNECTED,
	NDISWRAP_LINKSTATUS_AP_CHANGE,
	NDISWRAP_LINKSTATUS_AP_OUT_OF_RANGE,
	NDISWRAP_LINKSTATUS_AP_IN_RANGE,
	NDISWRAP_LINKSTATUS_ASSOC_FAILED,
	NDISWRAP_LINKSTATUS_AUTH_REQUEST,
	NDISWRAP_LINKSTATUS_LAST,
} ndiswraplinkstatus_t;

#define NDISWRAP_AUTH_REQUEST_REAUTH			0x01
#define NDISWRAP_AUTH_REQUEST_KEYUPDATE			0x02
#define NDISWRAP_AUTH_REQUEST_PAIRWISE_ERROR	0x06
#define NDISWRAP_AUTH_REQUEST_GROUP_ERROR		0x0E

typedef struct {
    long long   TransmittedFragments;
    long long   MulticastTransmittedFrames;
    long long   Faileds;
    long long   Retrys;
    long long   MultipleRetrys;
    long long   RTSSuccesses;
    long long   RTSFailures;
    long long   ACKFailures;
    long long   FrameDuplicates;
    long long   ReceivedFragments;
    long long   MulticastReceivedFrames;
    long long   FCSErrors;
} ndiswraplinkstats_t;

typedef struct {
	unsigned char Timestamp[8];
	unsigned short BeaconInterval;
	unsigned short Capabilities;
} ndiswrapfixed_ies;

typedef
__shimcall__
void
(*NDISWRAP_STATUS_HANDLER)(
    struct net_device *dev,
	ndiswraplinkstatus_t newstatus,
	ULONG flags,
	UCHAR BSSID[6]
    );

typedef
__shimcall__
void
(*NDISWRAP_XMITCOMPLETE_HANDLER)(
    struct net_device *dev,
	int err,
	struct sk_buff *skb
    );

typedef
__shimcall__
void
(*NDISWRAP_RESETCOMPLETE_HANDLER)(
    struct net_device *dev,
	int err,
	BOOLEAN addressingreset
    );

typedef
__shimcall__
void
(*NDISWRAP_HANG_HANDLER)(
    struct net_device *dev
    );

__shimcall__
int NdisWrapInit(void);
__shimcall__
void NdisWrapExit(void);

__shimcall__
ULONG NdisWrapEnter(void);
__shimcall__
void NdisWrapLeave(ULONG previousState);

__shimcall__
int NdisWrapCreateAdapter(POS_DEVNODE pDevNode, PCHAR ndisDriverPath, PNDIS_HANDLE pMiniportAdapterHandle);
__shimcall__
void NdisWrapDestroyAdapter(NDIS_HANDLE MiniportAdapterHandle);


#define NDISWRAP_CAPABILITY_WEP40	(1 << 0) /* aka WEP64 */
#define NDISWRAP_CAPABILITY_WEP104	(1 << 1) /* aka WEP128 */
#define NDISWRAP_CAPABILITY_WEP232	(1 << 2) /* aka WEP256 */
#define NDISWRAP_CAPABILITY_WEP		(NDISWRAP_CAPABILITY_WEP40|NDISWRAP_CAPABILITY_WEP104|NDISWRAP_CAPABILITY_WEP232)
#define NDISWRAP_CAPABILITY_TKIP	(1 << 3)
#define NDISWRAP_CAPABILITY_AES		(1 << 4)
#define NDISWRAP_CAPABILITY_WPA		(1 << 5)

__shimcall__
NDIS_STATUS
NdisWrapInitializeAdapter(
    IN  NDIS_HANDLE                    MiniportAdapterHandle,
	IN	NDISWRAP_STATUS_HANDLER        StatusHandler,
	IN	NDISWRAP_RX_HANDLER            RxHandler,
	IN	NDISWRAP_XMITCOMPLETE_HANDLER  XmitCompleteHandler,
	IN	NDISWRAP_RESETCOMPLETE_HANDLER ResetCompleteHandler,
	IN	NDISWRAP_HANG_HANDLER          HangHandler,
	OUT PULONG                         AdapterCapabilities
	);

__shimcall__
NDIS_STATUS
NdisWrapResetAdapter(
    IN  NDIS_HANDLE MiniportAdapterHandle,
    OUT PBOOLEAN    AddressingReset
    );

__shimcall__
VOID
NdisWrapNotifyAdapterRemoved(
    IN  NDIS_HANDLE             MiniportAdapterHandle
    );

__shimcall__
VOID
NdisWrapNotifyBusDeregister(OS_DEVTYPE Type);

__shimcall__
VOID
NdisWrapHaltAdapter(
    IN  NDIS_HANDLE             MiniportAdapterHandle
    );

__shimcall__
int NdisWrapStatusToErrno(NDIS_STATUS Status);

__shimcall__
VOID
NdisWrapReturnPacket(
	IN  PVOID		           PacketContext
    );

__shimcall__
NDIS_STATUS
NdisWrapGetVendorDesc(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  LPSTR                   vendordesc,
    IN  ULONG                   len
    );

__shimcall__
NDIS_STATUS
NdisWrapGetCurrentEthAddress(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    OUT PVOID                   InformationBuffer,
    OUT ULONG                   InformationBufferLength
    );

__shimcall__
NDIS_STATUS
NdisWrapSetCurrentEthAddress(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  PVOID                   InformationBuffer,
    IN  ULONG                   InformationBufferLength
    );

__shimcall__
NDIS_STATUS
NdisWrapGetLinkSpeed(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    OUT ULONG                   *Speed
	);

__shimcall__
NDIS_STATUS
NdisWrapGetLinkStats(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    OUT ndiswraplinkstats_t		*wrapLinkStats
	);

__shimcall__
NDIS_STATUS
NdisWrapGetMulticastListSize(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    OUT ULONG                   *ListSize
    );

__shimcall__
NDIS_STATUS
NdisWrapSetMulticastList(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  PUCHAR                  MulticastList,
    IN  ULONG                   Size
    );

__shimcall__
NDIS_STATUS
NdisWrapGetRtsThreshHold(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    OUT ULONG                   *Val
    );

__shimcall__
NDIS_STATUS
NdisWrapSetRtsThreshHold(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  ULONG                   Val
    );

__shimcall__
NDIS_STATUS
NdisWrapGetFragThreshHold(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    OUT ULONG                   *Val
    );

__shimcall__
NDIS_STATUS
NdisWrapSetFragThreshHold(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  ULONG                   Val
    );

__shimcall__
NDIS_STATUS
NdisWrapSetDesiredRates(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  PUCHAR                  DesiredRates,
    IN  ULONG					Len
	);

__shimcall__
NDIS_STATUS
NdisWrapGetDesiredRates(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    OUT PUCHAR                  DesiredRates,
    IN  ULONG					Len
	);

__shimcall__
NDIS_STATUS
NdisWrapGetSupportedRates(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    OUT PUCHAR                  SupportedRates,
    IN  ULONG					Len
	);

__shimcall__
NDIS_STATUS
NdisWrapGetNetworkType(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    OUT ndiswrapnetworktype_t	*NetworkTypeInUse
	);

__shimcall__
NDIS_STATUS
NdisWrapSetNetworkType(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  ndiswrapnetworktype_t	NetworkTypeInUse
	);

__shimcall__
NDIS_STATUS
NdisWrapSetPowerState(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  ndiswrappowerstate_t	PowerState
	);

__shimcall__
NDIS_STATUS
NdisWrapQueryPowerState(
	IN  NDIS_HANDLE             MiniportAdapterHandle,
	IN  ndiswrappowerstate_t	PowerState
	);

__shimcall__
NDIS_STATUS
NdisWrapSetPowerMode(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  ndiswrappowermode_t		pwrmode
	);

__shimcall__
NDIS_STATUS
NdisWrapGetPowerMode(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    OUT ndiswrappowermode_t		*pwrmode
	);

__shimcall__
NDIS_STATUS
NdisWrapSetTxPowerLevel(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  ULONG					TxPowerLevel
	);

__shimcall__
NDIS_STATUS
NdisWrapGetTxPowerLevel(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    OUT ULONG                  *TxPowerLevel
	);

__shimcall__
NDIS_STATUS
NdisWrapSetRssiTrigger(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  LONG					RssiTrigger
	);

__shimcall__
NDIS_STATUS
NdisWrapGetRssiTrigger(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    OUT LONG                   *RssiTrigger
	);

__shimcall__
NDIS_STATUS
NdisWrapGetNoiseLevel(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    OUT LONG                   *NoiseLevel
	);

__shimcall__
NDIS_STATUS
NdisWrapGetRssi(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    OUT LONG                   *Rssi
	);

__shimcall__
NDIS_STATUS
NdisWrapDisassociate(
    IN  NDIS_HANDLE             MiniportAdapterHandle
	);

__shimcall__
NDIS_STATUS
NdisWrapSetBSSIDListScan(
    IN  NDIS_HANDLE             MiniportAdapterHandle
	);

__shimcall__
NDIS_STATUS
NdisWrapSetInfraMode(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  ndiswrapiwmode_t        iwmode
    );

__shimcall__
NDIS_STATUS
NdisWrapSetConfiguration(
    IN NDIS_HANDLE              MiniportAdapterHandle,
    IN ULONG					BeaconPeriod,
    IN ULONG					ATIMWindow,
    IN ULONG					DSConfig,
    IN ULONG					HopPattern,
    IN ULONG					HopSet,
    IN ULONG					DwellTime
	);

__shimcall__
NDIS_STATUS
NdisWrapGetConfiguration(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    OUT PULONG					BeaconPeriod,
    OUT PULONG					ATIMWindow,
    OUT PULONG					DSConfig,
    OUT PULONG					HopPattern,
    OUT PULONG					HopSet,
    OUT PULONG					DwellTime
	);

__shimcall__
NDIS_STATUS
NdisWrapGetSSID(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  LPSTR                   essid,
    IN  ULONG                   len
    );


#define NDISWRAP_RANDOM_SSID NULL

__shimcall__
NDIS_STATUS
NdisWrapSetSSID(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  LPSTR                   essid
    );

__shimcall__
VOID
NdisWrapClearCachedSSID(
    IN  NDIS_HANDLE             MiniportAdapterHandle
    );

__shimcall__
NDIS_STATUS
NdisWrapSetBSSID(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    OUT PVOID                   InformationBuffer,
    IN  ULONG                   InformationBufferLength
    );

__shimcall__
NDIS_STATUS
NdisWrapGetBSSID(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    OUT PVOID                   InformationBuffer,
    IN  ULONG                   InformationBufferLength
    );

__shimcall__
NDIS_STATUS
NdisWrapSetAssocInfo(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    OUT PVOID                   InformationBuffer,
    IN  ULONG                   InformationBufferLength
    );

__shimcall__
NDIS_STATUS
NdisWrapGetAssocInfo(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  PVOID                   ReqIEs,
        PULONG                  nReqIEs,
    IN  PVOID                   RespIEs,
        PULONG                  nRespIEs
	);

__shimcall__
NDIS_STATUS
NdisWrapGetBSSIDList(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    OUT PVOID                   *ListPtr,
    OUT PULONG                  NumberOfEntries
    );

__shimcall__
VOID
NdisWrapFreeBSSIDList(
    IN  PVOID                   ListPtr
    );

__shimcall__
NDIS_STATUS
NdisWrapGetBSSIDListEntryMacAddress(
    IN  PVOID					ListPtr,
    IN  ULONG					Entry,
    OUT UCHAR                   MacAddress[6]
	);

__shimcall__
NDIS_STATUS
NdisWrapGetBSSIDListEntrySSID(
    IN  PVOID					ListPtr,
    IN  ULONG					Entry,
    OUT LPSTR					essid,
    IN  ULONG					len
	);

__shimcall__
NDIS_STATUS
NdisWrapGetBSSIDListEntryPrivacy(
    IN  PVOID					ListPtr,
    IN  ULONG					Entry,
    OUT PULONG					Privacy
	);

__shimcall__
NDIS_STATUS
NdisWrapGetBSSIDListEntryRssi(
    IN  PVOID					ListPtr,
    IN  ULONG					Entry,
    OUT PLONG					dBmRssi
	);

__shimcall__
NDIS_STATUS
NdisWrapGetBSSIDListEntryNetworkTypeInUse(
    IN  PVOID					ListPtr,
    IN  ULONG					Entry,
    OUT ndiswrapnetworktype_t	*NetworkTypeInUse
	);

__shimcall__
NDIS_STATUS
NdisWrapGetBSSIDListEntryConfiguration(
    IN  PVOID					ListPtr,
    IN  ULONG					Entry,
    OUT PULONG					BeaconPeriod,
    OUT PULONG					ATIMWindow,
    OUT PULONG					DSConfig,
    OUT PULONG					HopPattern,
    OUT PULONG					HopSet,
    OUT PULONG					DwellTime
	);

__shimcall__
NDIS_STATUS
NdisWrapGetBSSIDListEntryInfrastructureMode(
    IN  PVOID					ListPtr,
    IN  ULONG					Entry,
    OUT ndiswrapiwmode_t		*iwmode
	);

__shimcall__
NDIS_STATUS
NdisWrapGetBSSIDListEntrySupportedRates(
    IN  PVOID					ListPtr,
    IN  ULONG					Entry,
    OUT PUCHAR                  SupportedRates,
    IN  ULONG					Len
	);

__shimcall__
NDIS_STATUS
NdisWrapGetBSSIDListEntryIELength(
    IN  PVOID					ListPtr,
    IN  ULONG					Entry,
    OUT PLONG					IELength
	);

__shimcall__
NDIS_STATUS
NdisWrapGetBSSIDListEntryIEs(
    IN  PVOID					ListPtr,
    IN  ULONG					Entry,
    OUT PUCHAR                  IEs,
        PULONG					Len
	);

__shimcall__
NDIS_STATUS
NdisWrapEnableReceive(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  BOOLEAN                 Unicast,
    IN  BOOLEAN                 Broadcast,
    IN  BOOLEAN                 Multicast,
    IN  BOOLEAN                 AllMulticast,
    IN  BOOLEAN                 Promiscuous
    );

__shimcall__
NDIS_STATUS
NdisWrapSetWPA(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  PUCHAR					IEs,
    IN  ULONG					IELen,
	OUT ndiswrapauthmode_t		*authmode,
	OUT ndiswrapencryption_t	*encryption
	);

__shimcall__
NDIS_STATUS
NdisWrapSetWPAKey(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  ULONG					KeyIndex,
    IN  PUCHAR					KeyMaterial,
    IN  ULONG					KeyLength,
    IN  PUCHAR					KeyRSC,
    IN  ULONG					KeyRSCLength,
    IN  UCHAR                   BSSID[6],
    IN  BOOLEAN					SelectAsTxKey,
    IN  BOOLEAN					PairwiseKey,
    IN  BOOLEAN					Authenticator
	);

__shimcall__
NDIS_STATUS
NdisWrapWepReloadDefaults(
    IN  NDIS_HANDLE             MiniportAdapterHandle
	);

__shimcall__
NDIS_STATUS
NdisWrapSetWepKey(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  ULONG					KeyIndex,
    IN  PUCHAR					KeyMaterial,
    IN  ULONG					KeyLength,
    IN  BOOLEAN					SelectAsTxKey
	);

__shimcall__
NDIS_STATUS
NdisWrapSetEncryption(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  ndiswrapencryption_t	Type
	);

__shimcall__
NDIS_STATUS
NdisWrapGetEncryption(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    OUT ndiswrapencryption_t	*Type
	);

__shimcall__
NDIS_STATUS
NdisWrapSetPrivacyFilter(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  BOOLEAN					Enable
	);

__shimcall__
NDIS_STATUS
NdisWrapGetPrivacyFilter(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    OUT BOOLEAN					*Enabled
	);

__shimcall__
NDIS_STATUS
NdisWrapSetAuthMode(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  ndiswrapauthmode_t		AuthMode
	);

__shimcall__
NDIS_STATUS
NdisWrapGetAuthMode(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    OUT ndiswrapauthmode_t		*AuthMode
	);

__shimcall__
NDIS_STATUS
NdisWrapSetMachineName(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  PUCHAR					MachineName
	);

__shimcall__
NDIS_STATUS
NdisWrapTransmit(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  PVOID                   Data,
    IN  ULONG                   Len,
    IN  PVOID                   Context
    );

#endif // _DLDR_NDISWRAP_H_
