/*
 * Copyright (c) 2003-2004 Linuxant inc.
 *
 * NOTE: The use and distribution of this software is governed by the terms in
 * the file LICENSE, which is included in the package. You must read this and
 * agree to these terms before using or distributing this software.
 *
 */
#include "oscompat.h"
#include "typedefs.h"
#include "osservices.h"

#include <linux/slab.h>
#include <linux/vmalloc.h>
#include <linux/highmem.h>
#include <linux/pci.h>

//#define OS_DEBUG_MEM

__shimcall__
void *OsAllocate(unsigned size)
{
	PVOID pMem = kmalloc(size, (OsGetIrql() == OSIRQL_PASSIVE) ? GFP_KERNEL : GFP_ATOMIC);
	if(!pMem) {
		if(OsGetIrql() == OSIRQL_PASSIVE) {
			/* try vmalloc since kmalloc is generally limited to 64k */
			pMem = vmalloc(size);
			//printk(KERN_DEBUG "%s(%u): vmalloc returned %p\n", __FUNCTION__, size, pMem);
		}
	}

	if(pMem) {
#ifdef OS_DEBUG_MEM
		printk(KERN_DEBUG "%s(%u): %p\n", __FUNCTION__, size, pMem);
#if 0
		if((size == 21) || (size == 85))
			OsDumpStack();
#endif
#endif
		memset(pMem, 0, size); /* caller expects memory to be initialized to zero */
	} else {
		printk(KERN_ERR "%s(%u): failed, OsIrql=%d\n", __FUNCTION__, size, OsGetIrql());
	}

	return pMem;
}
 
__shimcall__
void OsFree(PVOID pMem)
{
	if(!pMem) {
		printk(KERN_ERR "%s: NULL\n", __FUNCTION__);
		return;
	}
	if((pMem >= (PVOID)VMALLOC_START) && (pMem < (PVOID)VMALLOC_END)) {
#ifdef OS_DEBUG_MEM
		printk(KERN_DEBUG "%s: vfree %p\n", __FUNCTION__, pMem);
#endif
		if(OsGetIrql() != OSIRQL_PASSIVE) {
			printk(KERN_WARNING "%s: calling vfree(%p) in wrong context, OsIrql=%d\n", __FUNCTION__, pMem, OsGetIrql());
		}
		vfree(pMem);
	} else {
#ifdef OS_DEBUG_MEM
		printk(KERN_DEBUG "%s: %p\n", __FUNCTION__, pMem);
#endif
		kfree(pMem);
	}
}

__shimcall__
void *OsDmaAllocate(HANDLE DevNode, unsigned size, unsigned long *physAddr)
{
	dma_addr_t dma_addr;
	void *virt_addr;

	virt_addr = pci_alloc_consistent((struct pci_dev *)DevNode, size, &dma_addr);

	*physAddr = dma_addr;

	return virt_addr;
}

__shimcall__
void OsDmaFree(HANDLE DevNode, unsigned size, void *vAddr, unsigned long physAddr)
{
	pci_free_consistent((struct pci_dev *)DevNode, size, vAddr, physAddr);
}
