/*
 * Copyright (c) 2003-2004 Linuxant inc.
 *
 * NOTE: The use and distribution of this software is governed by the terms in
 * the file LICENSE, which is included in the package. You must read this and
 * agree to these terms before using or distributing this software.
 *
 */

#include "oscompat.h"
#include "osservices.h"

#include <linux/pci.h>
#include <linux/slab.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <linux/interrupt.h>

__shimcall__
ULONG OsVirtToPhys(PVOID va)
{
#ifdef DLDR_TEST
	if (!IsKMallocMem(va) && va != OsMappedToVirt(va)) {
		OsErrorPrintf("%s: wrong mem type? va=%p virt_to_bus(va)=%lx OsMappedToVirt(va)=%p virt_to_bus(OsMappedToVirt(va)=%lx\n", __FUNCTION__, va, virt_to_bus(va), OsMappedToVirt(va), virt_to_bus(OsMappedToVirt(va)));
	}
#endif

	return(virt_to_bus(va));
}

__shimcall__
void *OsMappedToVirt(PVOID ma)
{
#if ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,19) )
	struct page *p = vmalloc_to_page(ma);

	if(p)
		return(void*)(((unsigned long)page_address(p)) | (((unsigned long)ma) & (PAGE_SIZE-1)));
	else
#endif
		return ma;
}

__shimcall__
VOID OsGetPCIDeviceResources(HANDLE DevNode, POS_PCI_DEV_RES pRes, int *nRes)
{
	struct pci_dev *pcidev = (struct pci_dev *)DevNode;
	POS_PCI_DEV_RES p = pRes;
	int i;

	memset(pRes, 0, sizeof(*pRes) * *nRes);

	if(pcidev->irq && (*nRes > 0)) {
		p->type = OSPCIRESTYPE_INTERRUPT;
		p->addr = pcidev->irq;
		p++;
	}

	for (i = 0; ((p - pRes) < *nRes) && (i < DEVICE_COUNT_RESOURCE); i++) {
		if(((pci_resource_flags(pcidev, i) & PCI_BASE_ADDRESS_SPACE) == PCI_BASE_ADDRESS_SPACE_MEMORY) && (pci_resource_start(pcidev, i) & PCI_BASE_ADDRESS_MEM_MASK) && (pci_resource_len(pcidev, i))) {
			p->type = OSPCIRESTYPE_MEMORY;
			p->addr = pci_resource_start(pcidev, i) & PCI_BASE_ADDRESS_MEM_MASK;
			p->len  = pci_resource_len(pcidev, i);
			p++;
			continue;
		}
		if(((pci_resource_flags(pcidev, i) & PCI_BASE_ADDRESS_SPACE) == PCI_BASE_ADDRESS_SPACE_IO) && (pci_resource_start(pcidev, i) & PCI_BASE_ADDRESS_IO_MASK) && (pci_resource_len(pcidev, i))) {
			p->type = OSPCIRESTYPE_PORT;
			p->addr = pci_resource_start(pcidev, i) & PCI_BASE_ADDRESS_IO_MASK;
			p->len  = pci_resource_len(pcidev, i);
			p++;
			continue;
		}
	}

	*nRes = p - pRes;
}

__shimcall__
VOID OsSetPciBusMaster(HANDLE DevNode)
{
	struct pci_dev *pcidev = (struct pci_dev *)DevNode;

	pci_set_master(pcidev);
}

__shimcall__
PVOID OsMapPhysMem(UINT32 dwPhysMemBase, UINT32 dwMemSize)
{
	PVOID mem;

	mem = ioremap_nocache(dwPhysMemBase, dwMemSize); /* XXX nocache */
	if(!mem) {
		printk(KERN_ERR "%s: ioremap(0x%lx, 0x%lx) failed\n", __FUNCTION__, dwPhysMemBase, dwMemSize);
	}
	//	printk("\n<HCF> Map %p (%lu) to %p\n\n", (void *)dwPhysMemBase, dwMemSize, (void *)mem);
	return mem;
}

__shimcall__
BOOL OsUnmapPhysMem(PVOID dwLinearAddr, UINT32 dwMemSize)
{
	if(dwLinearAddr != NULL)
		iounmap((void *)dwLinearAddr);
	return TRUE;
}

__shimcall__
BOOL OsHookInterrupt(HANDLE DevNode, __kernelcall__ OS_IRQRETURN_T (*isr)(int irq, void *devidp, void *unused), void* devid)
{
	struct pci_dev *pcidev = (struct pci_dev *)DevNode;
	const char *devname;
	POS_DEVNODE pDevNode = (POS_DEVNODE)pci_get_drvdata(pcidev);
	int err;

	if (pDevNode->hwModule && ((struct module *)pDevNode->hwModule)->name)
		devname = ((struct module *)pDevNode->hwModule)->name;
	else
		devname = (THIS_MODULE && THIS_MODULE->name) ? THIS_MODULE->name : "dldr";

    /* for efficiency we expect the isr function passed to have the same calling
       conventions and arguments as standard Linux interrupt handlers. */

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
	err = request_irq(pcidev->irq, (void (*)(int irq, void *devidp, struct pt_regs *))isr, SA_SHIRQ, devname, devid);
#else
    err = request_irq(pcidev->irq, (irqreturn_t (*)(int irq, void *devidp, struct pt_regs *))isr, SA_SHIRQ, devname, devid);
#endif
	if (err) {
		printk(KERN_ERR "%s: request_irq(dev=%s, irq=%d) failed err %d\n", __FUNCTION__, pcidev->slot_name, pcidev->irq, err);
		return FALSE;
	}

	return TRUE;
}

__shimcall__
void OsUnhookInterrupt(HANDLE DevNode, void *devid)
{
	struct pci_dev *pcidev = (struct pci_dev *)DevNode;

	if(pcidev && pcidev->irq) { /* IRQ 0? */
		free_irq(pcidev->irq, devid);
	}
}

__shimcall__
void
OsDisableInterrupt(HANDLE DevNode, BOOLEAN Sync)
{
	struct pci_dev *pcidev = (struct pci_dev *)DevNode;

	if(pcidev && pcidev->irq) { /* IRQ 0? */
		if(Sync)
			disable_irq(pcidev->irq);
		else
			disable_irq_nosync(pcidev->irq);
	}
}

__shimcall__
void
OsEnableInterrupt(HANDLE DevNode)
{
	struct pci_dev *pcidev = (struct pci_dev *)DevNode;

	if(pcidev && pcidev->irq) { /* IRQ 0? */
		enable_irq(pcidev->irq);
	}
}


#ifndef __i386__
__shimcall__
unsigned short OsIoMemReadw(void *addr)
{
	unsigned short val = readw(addr);
/*  	printk(KERN_DEBUG "OsIoMemReadw read 0x%x from 0x%x\n",val,addr);  */
	return val;
}

__shimcall__
unsigned int OsIoMemReadl(void *addr)
{
	unsigned int val = readl(addr);
/*  	printk("OsIoMemRead read 0x%x from 0x%x\n",val,addr); */
	return (val);
}

__shimcall__
void	OsIoMemWritew(void *addr, unsigned short wVal)
{
/*  	printk(KERN_DEBUG "OsIoMemWritew writes 0x%x to 0x%x\n",wVal,addr); */
	writew(wVal, addr);
}
 
 
__shimcall__
void	OsIoMemWritel(void *addr, unsigned int dwVal)
{
/*  	printk(KERN_DEBUG "OsIoMemWritel(%x, %x)\n",addr, dwVal); */
	writel(dwVal, addr);
}
#endif /* __i386__ */
 
 
__shimcall__
unsigned char OsIoPortReadb(unsigned int port)
{
	unsigned char val = inb(port);
/*  	printk(KERN_DEBUG "OsIoPortReadb read 0x%x from 0x%x\n",val,port);  */
	return val;
}
 
 
__shimcall__
unsigned short OsIoPortReadw(unsigned int port)
{
	unsigned short val = inw(port);
/*  	printk(KERN_DEBUG "OsIoPortReadw read 0x%x from 0x%x\n",val,port);  */
	return val;
}
 
__shimcall__
unsigned int OsIoPortReadl(unsigned int port)
{
	unsigned int val = inl(port);
/*  	printk(KERN_DEBUG "OsIoPortReadw read 0x%x from 0x%x\n",val,port);  */
	return val;
}

__shimcall__
void OsIoPortReadsb(unsigned int port, void * addr, unsigned long count)
{
	insb(port, addr, count);
}

__shimcall__
void OsIoPortReadsw(unsigned int port, void * addr, unsigned long count)
{
	insw(port, addr, count);
}

__shimcall__
void OsIoPortReadsl(unsigned int port, void * addr, unsigned long count)
{
	insl(port, addr, count);
}
 
 
__shimcall__
void	OsIoPortWriteb(unsigned int port, unsigned char bVal)
{
/*  	printk(KERN_DEBUG "OsIoPortWritew writes 0x%x to 0x%x\n",bVal,addr); */
	outb(bVal, port);
}
 
__shimcall__
void	OsIoPortWritew(unsigned int port, unsigned short wVal)
{
/*  	printk(KERN_DEBUG "OsIoPortWritew writes 0x%x to 0x%x\n",wVal,addr); */
	outw(wVal, port);
}
 
 
__shimcall__
void	OsIoPortWritel(unsigned int port, unsigned int dwVal)
{
/*  	printk(KERN_DEBUG "OsIoPortWritel(%x, %x)\n",addr, dwVal); */
	outl(dwVal, port);
}
 
__shimcall__
void	OsIoPortWritesb(unsigned int port, const void * addr, unsigned long count)
{
	outsb(port, addr, count);
}

__shimcall__
void	OsIoPortWritesw(unsigned int port, const void * addr, unsigned long count)
{
	outsw(port, addr, count);
}

__shimcall__
void	OsIoPortWritesl(unsigned int port, const void * addr, unsigned long count)
{
	outsl(port, addr, count);
}

__shimcall__
int OsPciReadConfigdw(void *pPciDev, unsigned char Offset, unsigned long *pVal)
{
	//printk("%s: %p Offset=%u pVal=%p\n", __FUNCTION__, pPciDev, (unsigned)Offset, pVal);
	return pci_read_config_dword(pPciDev, Offset, (u32*)pVal);
}
 
__shimcall__
int OsPciReadConfigw(void *pPciDev, unsigned char Offset, unsigned short *pVal)
{
	//printk("%s: %p Offset=%u pVal=%p\n", __FUNCTION__, pPciDev, (unsigned)Offset, pVal);
	return pci_read_config_word(pPciDev, Offset, pVal);
}
 
__shimcall__
int OsPciReadConfigb(void *pPciDev, unsigned char Offset, unsigned char *pVal)
{
	//printk("%s: %p Offset=%u pVal=%p\n", __FUNCTION__, pPciDev, (unsigned)Offset, pVal);
	return pci_read_config_byte(pPciDev, Offset, pVal);
}
 
__shimcall__
int OsPciWriteConfigdw(void *pPciDev, unsigned char Offset, unsigned long Val)
{
	//printk("%s: %p Offset=%u val=0x%lx\n", __FUNCTION__, pPciDev, (unsigned)Offset, (unsigned long)Val);
	return pci_write_config_dword(pPciDev, Offset, Val);
}
 
__shimcall__
int OsPciWriteConfigw(void *pPciDev, unsigned char Offset, unsigned short Val)
{
	//printk("%s: %p Offset=%u val=0x%lx\n", __FUNCTION__, pPciDev, (unsigned)Offset, (unsigned long)Val);
	return pci_write_config_word(pPciDev, Offset, Val);
}
 
__shimcall__
int OsPciWriteConfigb(void *pPciDev, unsigned char Offset, unsigned char Val)
{
	//printk("%s: %p Offset=%u val=0x%lx\n", __FUNCTION__, pPciDev, (unsigned)Offset, (unsigned long)Val);
	return pci_write_config_byte(pPciDev, Offset, Val);
}
