/*
 * Copyright (c) 2003-2004 Linuxant inc.
 *
 * NOTE: The use and distribution of this software is governed by the terms in
 * the file LICENSE, which is included in the package. You must read this and
 * agree to these terms before using or distributing this software.
 *
 */

#define OS_NOSTRINGREDEFS

#include "oscompat.h"
#include "osservices.h"

#include <linux/ctype.h>

__shimcall__
PVOID OsMemSet(PVOID pBuf, UINT8 c, UINT32 Count)
{
#if 0 // check for stack overflows:
	void *memstart = pBuf, *memend = pBuf + Count - 1;
	void *taskstart = current, *taskend = taskstart + sizeof(struct task_struct) - 1;

	if(((memstart >= taskstart) && (memstart <= taskend)) ||
		((memend >= taskstart) && (memend <= taskend)) ||
		((memstart < taskstart) && (memend > taskend))) {
		OsErrorPrintf("%s: %p c=%u Count=%lu taskstart=%p taskend=%p\n", __FUNCTION__, pBuf, (unsigned int)c, Count, taskstart, taskend);
		KDB_ENTER();
	}
#endif
    return memset(pBuf, c, Count);
}

__shimcall__
PVOID OsMemChr(void *s, int c, size_t n)
{
    return memchr(s, c, n);
}

__shimcall__
PVOID OsMemMove(PVOID pDest, const void *pSrc, UINT32 Count)
{
    return memmove(pDest, pSrc, Count);
}

__shimcall__
PVOID OsMemCpy(PVOID pDest, const void *pSrc, UINT32 Count)
{
    return memcpy(pDest, pSrc, Count);
}

__shimcall__
int OsMemCmp(const void *pBuff1, const void *pBuff2, UINT32 Count)
{
    return memcmp(pBuff1, pBuff2, Count);
}

__shimcall__
PVOID OsStrCpy(LPSTR szDest, LPCSTR szSrc)
{
    return strcpy(szDest, szSrc);
}

__shimcall__
PVOID OsStrnCpy(LPSTR szDest, LPCSTR szSrc, int MaxSize)
{
    return strncpy(szDest, szSrc, MaxSize);
}

__shimcall__
LPSTR OsStrCat(LPSTR szDest, LPCSTR szSrc)
{
    return strcat(szDest, szSrc);
}

__shimcall__
LPSTR OsStrnCat(LPSTR szDest, LPCSTR szSrc, int MaxSize)
{
    return strncat(szDest, szSrc, MaxSize);
}

__shimcall__
int OsStrCmp(LPCSTR szStr1, LPCSTR szStr2)
{
    return strcmp(szStr1, szStr2);
}

__shimcall__
int OsStrnCmp(LPCSTR szStr1, LPCSTR szStr2, UINT32 Count)
{
    return strncmp(szStr1, szStr2, Count);
}

__shimcall__
int OsStrCaseCmp(LPCSTR szStr1, LPCSTR szStr2)
{
	register signed char res;

	if (szStr1 == szStr2)
		return 0;

	while (1) {
		if ((res = toupper(*szStr1) - toupper(*szStr2++)) != 0 || !*szStr1++) {
			break;
		}
	}
	return res;
}

__shimcall__
int OsStrnCaseCmp(LPCSTR szStr1, LPCSTR szStr2, UINT32 Count)
{
	register signed char res = 0;

	if (szStr1 == szStr2)
		return 0;

	while (Count--) {
		if ((res = toupper(*szStr1) - toupper(*szStr2++)) != 0 || !*szStr1++) {
			break;
		}
	}
	return res;
}

__shimcall__
int OsStrLen(const char * string)
{
    return strlen(string);
}

__shimcall__
int OsToupper(int ch)
{
    return (ch >='a' && ch <= 'z') ? ch-'a'+'A' : ch;
}

__shimcall__
int OsTolower(int ch)
{
    return (ch >='A' && ch <= 'Z') ? ch-'A'+'a' : ch;
}

__shimcall__
int OsIsDigit(int ch)
{
    return (ch >='0' && ch <='9');
}

__shimcall__
int OsVSprintf(LPSTR buffer, LPCSTR format, va_list ap)
{
    return vsprintf(buffer, format, ap);
}

__shimcall__
int OsVSNprintf(LPSTR buffer, int n, LPCSTR format, va_list ap)
{
    return vsnprintf(buffer, n, format, ap);
}

__shimcall__
int OsSprintf(LPSTR buffer, LPCSTR format, ...)
{
    va_list   args;
    int    RetVal;

    va_start (args , format);
    RetVal = vsprintf(buffer,format,args);
    va_end (args);
    return RetVal;
}

__shimcall__
int OsSNprintf(LPSTR buffer, int n, LPCSTR format, ...)
{
    va_list   args;
    int    RetVal;

    va_start (args , format);
    RetVal = vsnprintf(buffer,n,format,args);
    va_end (args);
    return RetVal;
}

__shimcall__
int OsAtoi(LPCSTR szStr)
{
    LPCSTR pb = szStr;
	int neg, base, val;

	if(*pb == '-') {
		neg = 1;
		pb++;
	} else
		neg = 0;

	if(!strncmp(pb, "0x", 2)) {
		base = 16;
		pb += 2;
	} else
		base = 10;

	val = simple_strtoul(pb, NULL, base);;
	if(neg)
		val =  0 - val;

	return val;
}

__shimcall__
unsigned long OsStrtoul(const char *nptr, char **endptr, int base)
{
	return simple_strtoul(nptr, endptr, base);
}
