#! /usr/bin/perl
#
# Copyright (c) 2003-2004 Linuxant inc.
#
# NOTE: The use and distribution of this software is governed by the terms in
# the file LICENSE, which is included in the package. You must read this and
# agree to these terms before using or distributing this software.
# 

use lib qw(..); # to find our own CGI.pm
use CGI;
require "../dldrparser.pm";

my $cgi = new CGI;
my %driverConfig, @dev, %inst;
my $help, $strings;

sub readConfig {
	open(CONFIG, "includes/config.inc");
	while ( ($var,$val) = split(/=/, <CONFIG>) ) {
		chomp($val);
		$driverConfig{$var} = $val;
	}
	close(CONFIG);
}

sub dumpFile {
	my ($file) = @_;
	open(FILE, $file);
	while ( <FILE> ) {
		print;
	}
	close(FILE);
}

sub helpText {
	my ($help) = @_;

	print qq{<br><center><table cellspacing=4 cellpadding=6 width=90% bgcolor=#ffffee style="border-width:1px; border-style: solid; border-color: #bbbbaa;"><tr><td valign=top><img src="/images/info.gif"></td><td width=100%>$help</td></td></table></center>};
}

sub helpText_compact {
	my ($help) = @_;

	print qq{<center><table cellspacing=4 cellpadding=3 width=90% bgcolor=#ffffee style="border-width:1px; border-style: solid; border-color: #bbbbaa;"><tr><td valign=top><img src="/images/info.gif"></td><td width=100%><font size=-1>$help</font></td></td></table></center>};
}

sub printHeader {
	print "Content-type: text/html\n\n";
	dumpFile("includes/header.inc");
}

sub printMainHeader {
	print "Content-type: text/html\n\n";
	dumpFile("includes/mainhead.inc");
}

sub printFooter {
	dumpFile("includes/footer.inc");
}

sub nextSection {
        my $section, $line;

        while ( $line = <INF> ) {
                if ( $line =~ /^\[/ ) {
                        $section = $line; chomp($section);
                        $section =~ s/^\[//;
                        $section =~ s/\].*//;
                        $section =~ tr/A-Z/a-z/;
                        return $section;
                }
        }
        return 0;
}       

sub skipToSection {
        my ($section) = @_;

        seek(INF,0,0);
        $section =~ tr/A-Z/a-z/;
        while ( $nextsection = nextSection() ) {
                if ( $section eq $nextsection ) {
                        return $nextsection;
                }
        }
        return 0;
}

sub ltrim {
        my ($inp) = @_;
        $inp =~ s/^\s+//;
        return($inp);
}

sub rtrim {
        my ($inp) = @_;
        $inp =~ s/\s+$//;
        return ($inp);
}

sub trim {
        my ($inp) = @_;
        $inp=ltrim($inp);
        $inp=rtrim($inp);
        return($inp);
}

sub strip_quotes {
        my ($inp) = @_;
        $inp =~ s/\"//g;
        return($inp);
}

sub readStrings {
        my $line;

        skipToSection("strings");
        while ( $line = <INF> ) {
                if ( ($line =~ /^(\s)*;/) || ($line =~ /^(\s)*$/) ) {
                        next;
                }
                if ( ! ($line =~ /^\[/) ) {
                        chomp($line);
                        ($token, $string) = split(/=/, $line);
                        $token = trim($token);
						$token =~ tr/A-Z/a-z/;
                        $string = strip_quotes(trim($string));
                        $strings{"%" . $token . "%"} = $string;
                } else {
                        last;
                }
        }
        seek(INF,0,0);
}

sub getString {
        my ($string) = @_;

		if ( $string =~ /^%/ ) {
			$token=$string;
			$token=~ tr/A-Z/a-z/;
        	if ( exists($strings{$token}) ) {
           	     return $strings{$token};
        	} else {
           	     return $string;
        	}
		} else {
			return $string;
		}
}

sub getInfVersion {
	my ($driver, $parser) = @_;

	$infFile = "$driverConfig{'vardir'}/$driver.inf";

	if ( ! stat($infFile) ) {
		return "";
	} else {
		$parser = new dldrparser;

		$parser->readConfig;
		$parser->configFile($infFile);
		$parser->readStrings;
		$parser->skipToSection("Version");
		while ( $line = $parser->getLine ) {
			if ( $line =~ /^\s*Provider\s*=\s*(.*)/i ) {
				$provider = $line;
			}
			if ( $line =~ /^\s*DriverVer\s*=/i ) {
				$ver = $line;
			}
		}
		$provider =~ s/^\s*.*=\s*//;
		$provider =~ s/\s*$//;
		$ver =~ s/^\s*.*=\s*//;
		$ver =~ s/\s*$//;
		return (strip_quotes($parser->getString($provider)), $parser->getString($ver));
	}
}

sub printDriverTable {
	$found=0;
	opendir(DRV, "$driverConfig{'vardir'}");
	while ( $file = readdir(DRV) ) {
		if ( $file =~ /.*\.inf$/ ) {
			if ( $found eq 0 ) {
				print qq{<center><table border=0 width=90% noborder bgcolor=#ffefef style="border-width:1px; border-style: solid; border-color: #bbbbaa;"><tr bgcolor=#eedede><th>Driver</th><th>Vendor</th><th>Version</th><th>Delete</th></tr>};
				$found++;
			}
			($driver_name) = split(/\./, $file);
			($infMan, $infVersion) = getInfVersion($driver_name);
			print "<tr align=center><td>$driver_name</td><td>$infMan</td><td>$infVersion</td><td>[<a onClick='javascript:return confirm(\"Are you sure you want to delete $driver_name?\")' href='/cgi-bin/config.cgi?cmd=remove&config=$driver_name.inf';>X</a>]</td></tr>\n" if ($found > 0);
		}
	}
	closedir(DRV);
	print qq{</table></center>} if ($found > 0);
}

sub printConflictingModules {
	if(open(MODCONFLICTS, "<$driverConfig{'cnfdir'}/.modconflicts-" . `uname -r`)) {
		while(<MODCONFLICTS>) {
			($mod, $infs) = split;
			$infs =~ s/\.inf//g;
			$conflicting{$mod}=$infs;
		}
		close(MODCONFLICTS);

		if($#conflicting) {
			$help="";
			foreach $mod (keys %conflicting) {
				$help .= qq{Kernel module <code>$mod</code> overridden by <code>$conflicting{$mod}</code><br>};
				if(open(LSMOD, "/sbin/lsmod|")) {
					while(<LSMOD>) {
						($m, $rest) = split;
						if($m eq $mod) {
							$help .= qq{<strong>WARNING:</strong> module <code>$mod</code> still loaded; try &quot;<code>rmmod ${mod}</code>&quot; or reboot<br>};
							last;
						}
					}
					close(LSMOD);
				}
			}
			if($help) {
				helpText($help);
			}
		}
	}
}

sub printOptions {

	print qq{ <br><form method=post action='/cgi-bin/config.cgi'> };
	print qq{ <input type=hidden name=cmd value='saveoptions'> };
	print qq{ <table border=0 width=100% cellpadding=1 cellspacing=0> };

	$option_webconf="";
	if(open(WEBCONF, "<$driverConfig{'nvmdir'}/webconf")) {
		$option_webconf=<WEBCONF>;
		chomp $option_webconf;
		close(WEBCONF);
	}
	if(!$option_webconf) {
		$option_webconf="127.0.0.1:18020";
	}

	print qq{ <tr><td width=80%>Web configurator listening address:port</td><td align=right><input name=option_webconf value='$option_webconf' size=20></td></tr> };
	print qq{ <tr><th colspan=2> }; helpText_compact("Set to <code>off</code> to disable web configuration access completely, or <code>*:18020</code> to make it accessible on port 18020 from other hosts (potential security risk). The default setting is <code>127.0.0.1:18020</code>"); print qq{ </tr> };

	print qq{ </table> };
	print qq{ <table border=0 width=100% cellpadding=1 cellspacing=0> };
	print qq{ <tr><td><br></td></tr> };
	print qq{ <tr><td width=90%>Workaround for Windows drivers requiring large amounts of stack space</td><td align=right><select name=option_stackworkaround> };
	if (-e "$driverConfig{'nvmdir'}/workarounds/stack") {
		print qq{ <option value='1' selected>on</option><option value='0'>off</option> };
	} else {
		print qq{ <option value='1'>on</option><option value='0' selected>off</option> };
	}
	print qq{ </select></td></tr> };
	print qq{ <tr><th colspan=2> }; helpText_compact("The kernel stack size under Windows is 12K but usually much smaller (typically < 6.5K) under Linux. Certain drivers which use a lot of stack space can cause overflows and spurious crashes. This experimental workaround tries to attenuate the problem, but is not compatible with all configurations.<br>Ideally, a kernel with 16K stack size should be used. For your convenience, pre-compiled stk16 kernels for Fedora and patches to add CONFIG_16KSTACKS support are available from the <code>Download</code> section at <a href=\"http://www.linuxant.com/driverloader\" target=\"_blank\">http://www.linuxant.com/driverloader/</a>."); print qq{ </tr> };

	if(`uname -r` =~ /^2\.4\./) {
		print qq{ <tr><td><br></td></tr> };
		print qq{ <tr><td width=90%>Workaround for USB Interrupt issue under 2.4 kernels</td><td align=right><select name=option_usbintworkaround> };
		if (-e "$driverConfig{'nvmdir'}/workarounds/usbint") {
			print qq{ <option value='1' selected>on</option><option value='0'>off</option> };
		} else {
			print qq{ <option value='1'>on</option><option value='0' selected>off</option> };
		}
		print qq{ </select></td></tr> };
		print qq{ <tr><th colspan=2> }; helpText_compact("The USB subsystem of the 2.4 kernel automatically re-submits completed interrupt requests while Windows USB drivers expect them not to be. This discrepancy can lead to crashes or an unusable device. If you are having problems and use the 'usb-uhci' module as host controller, you could try switching to the 'uhci' module instead to see if there is a difference. If the problem persists, you could try to enable this experimental workaround. The recommended solution is to upgrade to a 2.6 kernel."); print qq{ </tr> };
	}

	print qq{ <tr><td align=right><br> };
	print qq{ <input type=button value='Cancel' onClick='javascript:document.location="/cgi-bin/config.cgi"'>};
	print qq{ <input type=submit value='Save'></td><td></td></tr></table></form>};
}

sub printDetectedCards {

	opendir(PROC, "/proc/driver/$driverConfig{'dldrname'}");
	@dev = grep { /^[a-z]+[0-9]+/ } readdir(PROC);
	closedir(PROC);

	foreach $device (@dev) {
		$inst{$device}{"netif"} = $device;

		# Get HW Instance
		open(PROCDRV, "/proc/driver/$driverConfig{'dldrname'}/$device/hwinst");
		$inst{$device}{"id"} = <PROCDRV>; chop($inst{$device}{"id"});
		close(PROCDRV);

		# Get MAC Address
		open(PROCDRV, "/proc/driver/$driverConfig{'dldrname'}/$device/hwaddr");
		$inst{$device}{"addr"} = <PROCDRV>; chop($inst{$device}{"addr"});
		close(PROCDRV);

		# Get driver name
		open(PROCDRV, "/proc/driver/$driverConfig{'dldrname'}/$device/drivername");
		$inst{$device}{"drivername"} = <PROCDRV>;
		chop($inst{$device}{"drivername"}); 
		close(PROCDRV);

		# Check to see if instance is already configured
		$license_ok{$device} = 0;
		if ( ! stat("$driverConfig{'cnfdir'}/$inst{$device}{'addr'}") ) {
			$inst{$device}{"status"} = "NOT CONFIGURED";
		} else {
			open(LIC, "/proc/driver/" . $driverConfig{'dldrname'} . "/" . $device . "/licstatus");
			$licStatus = <LIC>;
			close(LIC);
			if ( $licStatus =~ /^OK/ ) {
				if ( $licStatus =~ /Expires/ ) {
					$licStatus =~ s/.*\((.*)\).*/\1/;
					$inst{$device}{"status"} = "$licStatus";
					$license_ok{$device} = 1;
				} elsif ( $licStatus =~ /(OEM|ONLY)/ ) {
					$licStatus =~ s/.*\((.*)\).*/\1/;
					$inst{$device}{"status"} = "$licStatus";
					$license_ok{$device} = 1;
				} else {
					$inst{$device}{"status"} = "OK";
					$license_ok{$device} = 1;
				}
			} else {
				$inst{$device}{"status"} = "$licStatus";
			}
		}
	}

	if ( @dev > 0 ) {
		$nolicense = 0;
		print qq{<table border=0 width=100% cellspacing=0 cellpadding=4><tr style="font-weight: bold;"><td>Device</td><td align=center>Driver</td><td align=center>MAC Address</td><td align=center>License</td><td></td></tr>};

		foreach $dev (sort @dev) {
			($infMan, $infVersion) = getInfVersion($inst{$dev}{"drivername"});
			$configured = 0;
			print qq{<tr style="border-width: 1px; border-style: solid; border-color: #666666;"><td>$dev</td><td align=center><span onMouseOver="return escape('<table border=0 cellpadding=6><tr><td valign=top><center><img src=/images/info.gif></td><td>$infMan<br>$infVersion</td></tr></table>')">$inst{$dev}{"drivername"}</span></td><td align=center>$inst{$dev}{"addr"}</td><td align=center bgcolor=};
			if ( $license_ok{$dev} ) {
				$configured = 1;
				print qq{ #CCFFCC};
			} else {
				$nolicense++;
				print qq{ #FFCCCC};
			}
			print qq{><font size=-2>$inst{$dev}{"status"}</font></td><td align=right>};
			if ( $configured ne 1 ) {
				print qq{<img src="/images/blink.gif" align=middle width=14> };
			}
			print qq{<font size=-1>[<a href="/cgi-bin/parse.cgi?config=$inst{$dev}{"drivername"}.inf&hwinst=$inst{$dev}{'id'}&dev=$dev">Settings<a>]</font></td></tr>};
		}
		print qq{</table>};
		if ( $nolicense > 0 ) {
			if ( $nolicense eq 1 ) {
				$is = "is";
				$plural = "";
				$a = "a";
			} else {
				$is = "are";
				$plural = "s";
				$a = "";
			}
			helpText("There $is $nolicense device$plural without $a valid license$plural.<br><br>To install a license, click on [<i>Settings</i>] next to the device.");
		} else {
			helpText("All devices are licensed and configured.<br><br>To change license information or to modify advanced settings, click on [<i>Settings</i>] next to the device.<br><br>To add a new driver or update an existing one, click on [<a href='config.cgi?cmd=add'><i>Upload Windows Driver</i></a>].<br>");
		}
	} else {
		print qq{No devices found.<br>};

		$driver_found = 0; $help = "";
		opendir(DRV, "$driverConfig{'vardir'}");
		while ( $file = readdir(DRV) ) {
			if ( $file =~ /.*\.inf$/ ) {
				$driver_found++;
				($driver_name) = split(/\./, $file);
				($infMan, $infVersion) = getInfVersion($driver_name);
			}
		}
		closedir(DRV);

		if ( $driver_found > 0 ) {
			helpText("It seems that your device is either not present or not supported by the installed driver(s). Either:<br><br>1. Plug-in your device and click [<a href='/cgi-bin/config.cgi'>Refresh</a>] to retry, <b>OR</b><br>2. Upload a new driver using the [<i>Upload Windows Driver</i>] button below.<p>If your device still does not appear, please look at the <a href='dmesg.cgi#bottom' target='_blank'>Kernel Messages</a> for possible clues and check that no conflicting kernel modules are loaded.");
		} else {
			helpText("Please upload your Windows driver(s).");
		}
	}

	print "<br><center>";
	&printDriverTable;
	&printConflictingModules;
	print qq{<br><font size=-1>[<a href='config.cgi?cmd=add'>};
	print "Upload Windows Driver</a>]<br><br></font>";
}

sub addInterface {
	print qq{
		<center><br>
		<form method=post enctype="multipart/form-data"
			action="/cgi-bin/fileupload.cgi">
		INF File: <input type=file size=60 name="upfile0"><br>
	};
	helpText("Click on the <i>Browse</i> button and locate the driver .INF or .NTF file (but not the AUTORUN.INF) from your device's Windows XP driver package (this will usually be found in a subdirectory on the driver CD-ROM supplied with your device). Afterwards, click the <i>OK</i> button below to continue.");
	print qq{
		<br><input type=submit value=" OK ">
		<input type=button value=" Cancel " onClick="javascript:document.location='/cgi-bin/config.cgi'">
		</form>
		</center>
	};
}

sub newInf {
	my $oldmode, $infdata;

	$parser = new dldrparser;

	$parser->configFile($driverConfig{'vardir'} . "/" . $driverConfig{'dldrname'} . ".tmp");
	$basename  = $parser->getBaseName;
	$sysfile   = $parser->getSysName;
	@copyfiles = $parser->getSrcFiles;

	if ( ($basename eq "") || ($sysfile eq "") || ($copyfiles eq 0) ) {
		helpText("The file you have specified is not a valid INF/NTF file. Please click below to try again.");
		print qq{<br><center>[<a href="/cgi-bin/config.cgi?cmd=add">Try Again</a>]<br><br></center>};
		printFooter;
		exit;
	}

	$driverConfig{'driver'} = $basename;

	opendir(PROCDIR, "/proc/driver/$driverConfig{'dldrname'}");
	while ( $line = readdir(PROCDIR) ) {
		if ( ! ($line =~ /^\./) ) {
			open(DRVNAME, "/proc/driver/$driverConfig{'dldrname'}/$line/drivername");
			$bname = dldrparser::trim(<DRVNAME>);
			$inuse{$bname} = 1;
			$usedby{$bname} = $line;
			close(DRVNAME);
			open(HWADDR, "/proc/driver/$driverConfig{'dldrname'}/$line/hwaddr");
			$macaddr = dldrparser::trim(<HWADDR>);
			close(HWADDR);
			open(CNFFILE, "$driverConfig{'cnfdir'}/$macaddr");
			while (($sysfile = <CNFFILE>) && !($sysfile =~ /ServiceBinary/i)) {}
			($x,$sysfile) = split(/=/, $sysfile);
			$sysinuse{dldrparser::trim(dldrparser::strip_quotes($sysfile))} = 1;
			$sysusedby{dldrparser::trim(dldrparser::strip_quotes($sysfile))} = $line;
		}
	}
	closedir(PROCDIR);

	if ( (stat($driverConfig{'vardir'} . "/$basename.inf")) && ($inuse{$basename}) ) {
		helpText("The file you are trying to upload already exists in the system and is presently in use by interface <b>$usedby{$basename}</b>.<br>Please delete the driver from the main page and try again.<br><br>Click <a href='/cgi-bin/config.cgi'>HERE</a> to go back.");
		print "<br>";
		printFooter;
		exit;
	}

	for (@copyfiles) {
		if ( stat($driverConfig{'vardir'} . "/$_") && $sysinuse{$_} ) {
			helpText("One of the driver files from this INF file already exists in the system and is presently in use by interface <b>$sysusedby{$_}</b>.<br>Please delete the driver from the main page and try again.<br><br>Click <a href='/cgi-bin/config.cgi'>HERE</a> to go back.");
			print "<br>";
			printFooter;
			exit;
		}
	}

	if (! rename ($driverConfig{'vardir'} . "/" . $driverConfig{'dldrname'} . ".tmp", $driverConfig{'vardir'} . "/$basename.inf")) {
		helpText("Error renaming ". $driverConfig{'vardir'} . "/" . $driverConfig{'dldrname'} . ".tmp" . " to ". $driverConfig{'vardir'} . "/$basename.inf" . ": ". $!);
		print qq{<br><center>[<a href="/cgi-bin/config.cgi?cmd=add">Try Again</a>]<br><br></center>};
		printFooter;
		exit;
	}

	if ( @copyfiles > 1 ) {
		$this   = "These";
		$is     = "are";
		$plural = "s";
	} else {
		$this   = "This";
		$is     = "is";
		$plural = "";
	}

	print qq{
		<center><br>
		<form method=post enctype="multipart/form-data" action="/cgi-bin/fileupload.cgi">
		<input type=hidden name=driver value="$driverConfig{'driver'}">
		The following file$plural $is required:<br><br><table border=0 cellpadding=4>
	};

	$filenum=0;
	foreach $file (@copyfiles) {
		print qq{<input type=hidden name=filename$filenum value="$file">
		<tr><td align=center><b>$file</b></td><td><input type=file size=60 name="upfile$filenum"></td></tr>};
		$filenum++;
	}

	print "</table><br>";

	helpText("Click on the corresponding <i>Browse</i> button to locate the above file$plural from your Windows XP driver.<br><br>$this will usually be found in the same directory as the .INF or .NTF file you have just specified.<br>Click the <i>OK</i> button below to continue.");
	print qq{
		<br><input type=submit value=" OK ">
		<input type=button value=" Cancel " onClick="javascript:document.location='/cgi-bin/config.cgi'">
		</form>
	};
}

sub removeConfig {
	my($file) = @_;
	$parser = new dldrparser;

	if ( ! stat($driverConfig{'vardir'} . "/$file") ) {
		return;
	}

	$parser->configFile($driverConfig{'vardir'} . "/$file");
	$basename  = $parser->getBaseName;
	$sysfile   = $parser->getSysName;
	@copyfiles = $parser->getSrcFiles;

	foreach $delfile (@copyfiles) {
		unlink $driverConfig{'vardir'} . "/$delfile" || die "Can not unlink file $driverConfig{'vardir'}/$delfile.";
	}
	unlink $driverConfig{'vardir'} . "/$file";

	$need_reload = 0;
	opendir(PROCDIR, "/proc/driver/$driverConfig{'dldrname'}");
	while ( $line = readdir(PROCDIR) ) {
		if ( ! ($line =~ /^\./) ) {
			open(DRVNAME, "/proc/driver/$driverConfig{'dldrname'}/$line/drivername");
			$bname = dldrparser::trim(<DRVNAME>);
			if ( "$bname.inf" eq "$file" ) {
				open(PROCDRV, "/proc/driver/$driverConfig{'dldrname'}/$line/hwinst");
				$hwinstfile = dldrparser::trim(<PROCDRV>);
				close(PROCDRV);
				$mainfile = readlink("$driverConfig{'cnfdir'}/$hwinstfile");
				unlink "$driverConfig{'cnfdir'}/$hwinstfile";
				unlink "$driverConfig{'cnfdir'}/$mainfile";
				$need_reload = 1;
			}
			close(DRVNAME);
		}
	}
	closedir(PROCDIR);

	if ( $need_reload ) {
		system($driverConfig{'dldrabbr'} . "stop >/dev/null 2>&1");
		system($driverConfig{'dldrabbr'} . "config --info >/dev/null 2>&1");
	}
}

################################################################################

&readConfig;

if ( defined($cgi->param('cmd')) && ($cgi->param('cmd') ne "reload") && ($cgi->param('cmd') ne "remove") && ($cgi->param('cmd') ne "saveoptions") ) {
	&printHeader;
} else {
	&printMainHeader;
}

if ( defined($cgi->param('cmd')) ) {

	if ( $cgi->param('cmd') eq "add" ) {
		&addInterface;
	}

	elsif ( $cgi->param('cmd') eq "newFile" ) {
		if ( $cgi->param('filetype') eq "INF" ) {
			&newInf;
		}
		else {
			helpText("File type not recognized.<br>Must be an .INF or .NTF file.");
			print qq{<br><center>[<a href="/cgi-bin/config.cgi?cmd=add">Try Again</a>]<br><br></center>};
			&printFooter;
			exit;
		}
	}

	elsif ( $cgi->param('cmd') eq "remove" ) {
		&removeConfig($cgi->param('config'));
		&printDetectedCards;
	}

	elsif ( $cgi->param('cmd') eq "reload" ) {
		system($driverConfig{'dldrabbr'} . "stop >/dev/null 2>&1");
		system($driverConfig{'dldrabbr'} . "config --info >/dev/null 2>&1");
		&printDetectedCards;
	}

	elsif ( $cgi->param('cmd') eq "options" ) {
		&printOptions;
	}

	elsif ( $cgi->param('cmd') eq "saveoptions" ) {

		$cfgargs="";

		if ($cgi->param('option_stackworkaround')) {
			$cfgargs .= " --enable-workaround=stack";
		} else {
			$cfgargs .= " --disable-workaround=stack";
		}

		if ($cgi->param('option_usbintworkaround')) {
			$cfgargs .= " --enable-workaround=usbint";
		} else {
			$cfgargs .= " --disable-workaround=usbint";
		}

		if ($cgi->param('option_webconf')) {
			$cfgargs .= " --webconf=\"" . $cgi->param('option_webconf') . "\"";
		}

		system("(sleep 1; " . $driverConfig{'dldrabbr'} . "config --auto" . $cfgargs . ") >/dev/null 2>&1 &");

		&printDetectedCards;
	}

} else {
	&printDetectedCards;
}

&printFooter;
